if (typeof(AppMobiInit) != 'object')
    AppMobiInit = {};

/**
 * This represents the AppMobi API itself, and provides a global namespace for accessing
 * information about the state of AppMobi.
 * @class
 */
AppMobi = {
    queue: {
        ready: true,
        commands: [],
        timer: null
    },
    _constructors: [],
    jsVersion: '3.4.0'
};

/**
 * Boolean flag indicating if the AppMobi API is available and initialized.
 */
AppMobi.isnative = true;
AppMobi.isxdk = false;
AppMobi.istest = false;
AppMobi.ischrome = false;
AppMobi.isfacebook = false;
AppMobi.isweb = false;
AppMobi.isphone = false;
AppMobi.istablet = false;
AppMobi.isamazon = false;
AppMobi.isgoogle = false;
AppMobi.isintel = false;
AppMobi.ismozilla = false;
AppMobi.isnook = false;
AppMobi.iswp8 = false;
AppMobi.iswin8 = false;
AppMobi.available = false;
AppMobi.oauthAvailable = false;
AppMobi.paymentsAvailable = false;
AppMobi.updateAvailable = false;
AppMobi.updateMessage = "";
AppMobi.app = "";
AppMobi.release = "";
AppMobi.package = "";
AppMobi.webRoot = "";

AppMobi.cookies = {};
AppMobi.mediacache = [];
AppMobi.notifications = [];
AppMobi.people = [];
AppMobi.picturelist = [];
AppMobi.recordinglist = [];
AppMobi.playingtrack = {};

/**
 * Add an initialization function to a queue that ensures it will run and initialize
 * application constructors only once AppMobi has been initialized.
 * @param {Function} func The function callback you want run once AppMobi is initialized
 */
AppMobi.addConstructor = function(func) {
    var state = document.readyState;
    if (state == 'loaded' || state == 'complete')
        func();
    else
        AppMobi._constructors.push(func);
};

(function() {
	if(AppMobi.initialize==undefined) {
		AppMobi.initialize = function() {
		    var state = document.readyState;
			//console.log("In AppMobi.initialize, state:" + state);
			
			//pageIsFinished is set when webview.onPageFinished() is called
			//the following initialization is to protect old clients that do not have this feature implemented
			try{
				var pageIsFinished = AppMobiDevice.isPageFinished();
			}catch(e){}
			if(undefined==pageIsFinished)pageIsFinished = true;
			
			if (pageIsFinished && (state == 'loaded' || state == 'complete')) {
				//run constructors
				while (AppMobi._constructors.length > 0) {
					var constructor = AppMobi._constructors.shift();
					try {
						constructor();
					} catch(e) {
//						if (typeof(AppMobi.debug['log']) == 'function')
//							AppMobi.debug.log("Failed to run constructor: " + AppMobi.debug.processMessage(e));
//						else
//							alert("Failed to run constructor: " + e.message);
					}
				}
				
				setTimeout('AppMobi.device.messagePump();', 150);	
				
				// all constructors run, now fire the deviceready event
				var js = new String(AppMobiDevice.injectJSBeforeReady());
				eval(js.toString());
				var e = document.createEvent('Events');
				e.initEvent('appMobi.device.ready',true,true);
				document.dispatchEvent(e);
			} else {
			    setTimeout("AppMobi.initialize()", 100);
			}
		};
	}
	setTimeout("AppMobi.initialize()", 100);
})();

/**
 * Execute a AppMobi command in a queued fashion, to ensure commands do not
 * execute with any race conditions, and only run when AppMobi is ready to
 * recieve them.
 * @param {String} command Command to be run in AppMobi, e.g. "ClassName.method"
 * @param {String[]} [args] Zero or more arguments to pass to the method
 */
AppMobi.exec = function() {
	//compatibility placeholder on android - needed for iphone
};

/**
 * Internal function used to dispatch the request to AppMobi.  This needs to be implemented per-platform to
 * ensure that methods are called on the phone in a way appropriate for that device.
 * @private
 */
AppMobi.run_command = function() {
	//compatibility placeholder on android - needed for iphone
};

/**
 * A utility function to generate a stack trace for debugging purposes 
 * @private
 */
AppMobi.generateStackTrace = function() {
	var e = new Error('dummy');
	var stack = e.stack.replace(/^[^\(]+?[\n$]/gm, '')
		.replace(/^\s+at\s+/gm, '')
		.replace(/^Object.<anonymous>\s*\(/gm, '{anonymous}()@')
		.split('\n');
	console.log(stack);	
}

/**
 * This class contains acceleration information
 * @constructor
 * @param {Number} x The force applied by the device in the x-axis.
 * @param {Number} y The force applied by the device in the y-axis.
 * @param {Number} z The force applied by the device in the z-axis.
 * @param {boolean} doRotate If true, rotate axes based on device rotation.
 */
AppMobi.Acceleration = function(x, y, z, doRotate) {
	if(doRotate) {
		var orientation = AppMobi.device.orientation;
		if(orientation==0) {
		//portrait
		} else if (orientation==90) {
		//landscape left
			var temp = y, y = -x, x = temp;
		} else if (orientation==180) {
		//upside-down portrait
			x = -x, y = -y;
		} else if (orientation==-90) {
		//landscape right
			var temp = x, x = -y, y = temp;
		}
	}
	/**
	 * The force applied by the device in the x-axis.
	 */
	this.x = x;
	/**
	 * The force applied by the device in the y-axis.
	 */
	this.y = y;
	/**
	 * The force applied by the device in the z-axis.
	 */
	this.z = z;
	/**
	 * The time that the acceleration was obtained.
	 */
	this.timestamp = new Date().getTime();
}

// Define object for receiving acceleration values
AppMobi._accel = new AppMobi.Acceleration(0, 0, 0, false);

/**
 * This class specifies the options for requesting acceleration data.
 * @constructor
 */
AppMobi.AccelerationOptions = function() {
	this.frequency = 500;
	this.adjustForRotation = false;
}

/**
 * This class provides access to device accelerometer data.
 * @constructor
 */
AppMobi.Accelerometer = function() {
}

/**
 * Asynchronously acquires the current acceleration.
 * @param {Function} successCallback The function to call when the acceleration
 * data is available
 * @param {AccelerationOptions} options The options for getting the accelerometer data
 * such as timeout.
 */
AppMobi.Accelerometer.prototype.getCurrentAcceleration = function(successCallback, options) {
	// If the acceleration is available then call success
	// If the acceleration is not available then call error

	//validate options object
	var _options = new AppMobi.AccelerationOptions();
	if(typeof(options)=="object"){
		if(typeof(options.adjustForRotation)=="boolean") _options.adjustForRotation = options.adjustForRotation;
	}
	// Created for iPhone, Iphone passes back _accel obj litteral
	if (typeof successCallback == "function") {
		var accel = new AppMobi.Acceleration(AppMobi._accel.x,AppMobi._accel.y,AppMobi._accel.z, _options.adjustForRotation);
		successCallback(accel);
	}
}

/**
 * Asynchronously acquires the acceleration repeatedly at a given interval.
 * @param {Function} successCallback The function to call each time the acceleration
 * data is available
 * @param {AccelerationOptions} options The options for getting the accelerometer data
 * such as timeout.
 */

AppMobi.Accelerometer.prototype.watchAcceleration = function(successCallback, options) {
	//validate options object
	var _options = new AppMobi.AccelerationOptions();
	if(typeof(options)=="object"){
		var parsedFreq = parseInt(options.frequency);
		if(typeof(parsedFreq)=="number" && !isNaN(parsedFreq)) {
			_options.frequency = parsedFreq<25?25:parsedFreq;
		}
		if(typeof(options.adjustForRotation)=="boolean") _options.adjustForRotation = options.adjustForRotation;
	}
	AppMobiAccelerometer.start(_options.frequency);
	AppMobi.accelerometer.getCurrentAcceleration(successCallback, _options);
	return setInterval(function() {
		AppMobi.accelerometer.getCurrentAcceleration(successCallback, _options);
	}, _options.frequency);
}

/**
 * Clears the specified accelerometer watch.
 * @param {String} watchId The ID of the watch returned from #watchAcceleration.
 */
AppMobi.Accelerometer.prototype.clearWatch = function(watchId) {
	AppMobiAccelerometer.stop();
	clearInterval(watchId);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.accelerometer == "undefined") AppMobi.accelerometer = new AppMobi.Accelerometer();
});

/**
 * This class provides access to the appMobiContacts
 * @constructor
 */

AppMobi.Contacts = function() {
};

AppMobi.Contacts.prototype.getContactList = function() {
	var contacts = [];
	for(var person in AppMobi.people) {
		contacts.push(AppMobi.people[person].id);
	}
	return contacts;
};

AppMobi.Contacts.prototype.getContactData = function(id) {
	var local = null;
	for(var person in AppMobi.people) {
		if(id==AppMobi.people[person].id) {
			local = {};
			local.id = AppMobi.people[person].id;
			local.first = AppMobi.people[person].first;
			local.last = AppMobi.people[person].last;
			local.name = AppMobi.people[person].name;
			local.phones = AppMobi.people[person].phones;
			local.emails = AppMobi.people[person].emails;
			local.addresses = AppMobi.people[person].addresses;
			break;
		}
	}
	return local;
};

AppMobi.Contacts.prototype.getContacts = function() {
	AppMobiContacts.getContacts();
}

AppMobi.Contacts.prototype.addContact = function(first, last, street, city, state, zip, country, phone, email) {    
    AppMobiContacts.addContact(first, last, street, city, state, zip, country, phone, email);
}

AppMobi.Contacts.prototype.chooseContact = function() {
    AppMobiContacts.chooseContact();
}

AppMobi.Contacts.prototype.editContact = function(contactID) {
   AppMobiContacts.editContact(contactID);
}

AppMobi.Contacts.prototype.removeContact = function(contactID) {
    AppMobiContacts.removeContact(contactID);
}

AppMobi.addConstructor(function() {
	if (typeof AppMobi.contacts == "undefined") AppMobi.contacts = new AppMobi.Contacts();
});

AppMobi.Advertising = function() {
};

AppMobi.Advertising.prototype.getAd = function(name, path, callbackId) {
	//path is the appconfig url
	//callbackId is a unique id that is used by the appmobi ad framework
	AppMobiAdvertising.getAd(name, path, callbackId);
};

AppMobi.Advertising.prototype.runPromotion = function(appname, storeurl, promotion, protocol, package, query) {
    if( typeof( appname ) != "string" || appname.length == 0 ||
        typeof( storeurl ) != "string" || storeurl.length == 0 ||
        typeof( promotion ) != "string" || promotion.length == 0 ||
        typeof( protocol ) != "string" || protocol.length == 0 ||
        typeof( package ) != "string" || package.length == 0 )
		throw(new Error("Error: AppMobi.advertising.runPromotion, appname, storeurl, promotion, protocol and package are required."));

	if( query == undefined || query.length == 0 ) query = "";
    
	AppMobiAdvertising.runPromotion(appname, storeurl, promotion, protocol, package, query);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.advertising == "undefined") AppMobi.advertising = new AppMobi.Advertising();
});

AppMobi.AudioInfo = function(currentTime, duration) {
	this.currentTime = currentTime;
	this.duration = duration;
};

/**
 * This class provides access to the appMobiPlayer
 * @constructor
 */
AppMobi.Player = function() {
	this.audioInfo = new AppMobi.AudioInfo(0, 0);
};

AppMobi.Player.prototype.show = function() {
    AppMobiPlayer.show();
};

AppMobi.Player.prototype.hide = function() {
    AppMobiPlayer.hide();
};

AppMobi.Player.prototype.playPodcast = function(strPodcastURL) {
    AppMobiPlayer.playPodcast(strPodcastURL);
};

AppMobi.Player.prototype.startStation = function(strNetStationID, boolResumeMode, boolShowPlayer) {
    AppMobiPlayer.startStation(strNetStationID, boolResumeMode, boolShowPlayer);
};

AppMobi.Player.prototype.playSound = function(strRelativeFileURL) {
    AppMobiPlayer.playSound( strRelativeFileURL);
};

AppMobi.Player.prototype.loadSound = function(strRelativeFileURL, count) {
	if( typeof( count ) != "number" ) count = 1;
	var new_fn = true;
    try { AppMobiPlayer.loadSound( strRelativeFileURL, count); } catch(e) { new_fn = false; }
    if( new_fn == false ) AppMobiPlayer.loadSound( strRelativeFileURL );
};

AppMobi.Player.prototype.unloadSound = function(strRelativeFileURL) {
    AppMobiPlayer.unloadSound( strRelativeFileURL);
};

AppMobi.Player.prototype.unloadAllSounds = function() {
    AppMobiPlayer.unloadAllSounds();
};

AppMobi.Player.prototype.startAudio = function(strRelativeFileURL, boolLoop) {
	if(boolLoop == null || boolLoop == undefined) boolLoop = false;
    AppMobiPlayer.startAudio(strRelativeFileURL, boolLoop);
};

AppMobi.Player.prototype.toggleAudio = function() {
    AppMobiPlayer.toggleAudio();
};

AppMobi.Player.prototype.pauseAudio = function() {
    AppMobiPlayer.pauseAudio();
};

AppMobi.Player.prototype.resumeAudio = function() {
    AppMobiPlayer.resumeAudio();
};

AppMobi.Player.prototype.stopAudio = function() {
    AppMobiPlayer.stopAudio();
};

AppMobi.Player.prototype.setAudioVolume = function(volume) {
    AppMobiPlayer.setAudioVolume(volume);
};

AppMobi.Player.prototype.setAudioCurrentTime = function(time) {
    AppMobiPlayer.setAudioCurrentTime(time);
};

AppMobi.Player.prototype.getAudioCurrentTime = function(successCallback) {
	if (typeof successCallback == "function") {
		successCallback(AppMobi.player.audioInfo);
	}
};

AppMobi.Player.prototype.watchAudioCurrentTime = function(successCallback, frequency) {
	AppMobiPlayer.startUpdatingAudioTime(frequency);
	return setInterval(function() {
		AppMobi.player.getAudioCurrentTime(successCallback);
	}, frequency);
};

AppMobi.Player.prototype.clearAudioCurrentTimeWatch = function(watchId) {
	AppMobiPlayer.stopUpdatingAudioTime();
	clearInterval(watchId);
};

AppMobi.Player.prototype.play = function() {
    AppMobiPlayer.play();
};

AppMobi.Player.prototype.pause = function() {
    AppMobiPlayer.pause();
};

AppMobi.Player.prototype.stop = function() {
    AppMobiPlayer.stop();
};

AppMobi.Player.prototype.volume = function(iPercentage) {
    AppMobiPlayer.volume(iPercentage);
};

AppMobi.Player.prototype.rewind = function() {
    AppMobiPlayer.rewind();
};

AppMobi.Player.prototype.ffwd = function() {
    AppMobiPlayer.ffwd();
};

AppMobi.Player.prototype.setColors = function(strBackColor, strFillColor, strDoneColor, strPlayColor) {
    AppMobiPlayer.setColors(strBackColor, strFillColor, strDoneColor, strPlayColor);
};

AppMobi.Player.prototype.setPosition = function(portraitX, portraitY, landscapeX, landscapeY) {
    AppMobiPlayer.setPosition(portraitX, portraitY, landscapeX, landscapeY);
};

AppMobi.Player.prototype.startShoutcast = function(strStationURL, boolShowPlayer) {
    AppMobiPlayer.startShoutcast(strStationURL, boolShowPlayer);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.player == "undefined") AppMobi.player = new AppMobi.Player();
});

/**
 * This class provides access to the appMobiStats
 * @constructor
 */
AppMobi.Stats = function() {
};

AppMobi.Stats.prototype.logEvent = function(message) {
	throw(new Error("Error: AppMobi.stats.logEvent is deprecated, use AppMobi.analytics.logPageEvent."));
};

AppMobi.addConstructor(function() {
	if (typeof AppMobi.stats == "undefined") AppMobi.stats = new AppMobi.Stats();
});

/**
 * This class provides access to the appMobiAnalytics
 * @constructor
 */
AppMobi.Analytics = function() {
};

AppMobi.Analytics.prototype.logPageEvent = function(page, query, status, method, bytes, referrer) {
	if(page==undefined || page.length==0)
		throw(new Error("Error: AppMobi.analytics.logPageEvent, No page specified."));
	
	if( query == undefined || query.length == 0 ) query = "-";
	if( status == undefined || status.length == 0 ) status = "200";
	if( method == undefined || method.length == 0 ) method = "GET";
	if( bytes == undefined || bytes.length == 0 ) bytes = "0";
	if( referrer == undefined || referrer.length == 0 ) referrer = "index.html";
	
	AppMobiAnalytics.logPageEvent(page, query, status, method, bytes, referrer);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.analytics == "undefined") AppMobi.analytics = new AppMobi.Analytics();
});

/**
 * This class provides access to the appMobiFile
 * @constructor
 */
AppMobi.File = function() {
};

AppMobi.File.prototype.uploadToServer = function(localURL, uploadURL, foldername, mimetype, updateCallback) {
	if( (localURL == undefined || localURL=='') || (uploadURL == undefined || uploadURL=='') )
	{
		throw(new Error("Error: AppMobi.file.uploadToServer has the following required parameters: localURL, uploadURL."));
	}
	if(updateCallback != null && typeof(updateCallback)!="string") throw(new Error("Error: AppMobi.file.uploadToServer updateCallback parameter must be a string which is the name of a function"));
	if(mimetype == null) mimetype = "text/plain";
	
	AppMobiFile.uploadToServer(localURL, uploadURL, foldername, mimetype, updateCallback);
};

AppMobi.File.prototype.cancelUpload = function() {
	AppMobiFile.cancelUpload();
};

AppMobi.addConstructor(function() {
	if (typeof AppMobi.file == "undefined") AppMobi.file = new AppMobi.File();
});


/**
 * This class provides access to the appMobiCamera
 * @constructor
 */
AppMobi.Camera = function() {
};

AppMobi.Camera.prototype.takePicture = function(quality, saveToLib, picType) {
	if(quality == undefined || quality == null)
		quality = 70; // default
	else if((quality<1) || (quality>100))
		throw(new Error("Error: AppMobi.camera.takePicture, quality must be between 1-100."));
	
	if(saveToLib == undefined || saveToLib == null) 
		saveToLib = true;
	
	if(typeof(picType) == "undefined" || picType == null) 
		picType = "jpg";
	else {
		if(typeof(picType) != "string")
			throw(new Error("Error: AppMobi.camera.takePicture, picType must be a string."));
		if((picType.toLowerCase() != "jpg") && (picType.toLowerCase() != "png"))
			throw(new Error("Error: AppMobi.camera.takePicture, picType must be 'jpg' or 'png'."));
	}
	AppMobiCamera.takePicture(quality, saveToLib, picType);
};

AppMobi.Camera.prototype.takeFrontPicture = function(quality, saveToLib, picType) {
	if(quality == undefined || quality == null)
		quality = 70; // default
	else if((quality<1) || (quality>100))
		throw(new Error("Error: AppMobi.camera.takeFrontPicture, quality must be between 1-100."));

	if(saveToLib == undefined || saveToLib == null) 
		saveToLib = true;

	if(typeof(picType) == "undefined" || picType == null) 
		picType = "jpg";
	else {
		if(typeof(picType) != "string")
			throw(new Error("Error: AppMobi.camera.takeFrontPicture, picType must be a string."));
		if((picType.toLowerCase() != "jpg") && (picType.toLowerCase() != "png"))
			throw(new Error("Error: AppMobi.camera.takeFrontPicture, picType must be 'jpg' or 'png'."));
	}
	AppMobiCamera.takePicture(quality, saveToLib, picType);
};

AppMobi.Camera.prototype.importPicture = function() {
	AppMobiCamera.importPicture();
};

AppMobi.Camera.prototype.deletePicture = function(picURL) {
	if(picURL == undefined || picURL == null) 
		throw(new Error("Error: AppMobi.camera.deletePicture, call with a picURL"));
	if(typeof(picURL) != "string")
		throw(new Error("Error: AppMobi.camera.deletePicture, picURL must be a string."));

	AppMobiCamera.deletePicture(picURL);
};

AppMobi.Camera.prototype.clearPictures = function() {
	AppMobiCamera.clearPictures();
};

AppMobi.Camera.prototype.getPictureList = function() {
	var list = [];
	for(var picture in AppMobi.picturelist) {
		list.push(AppMobi.picturelist[picture]);
	}
	return list;
}

AppMobi.Camera.prototype.getPictureURL = function(filename) {
	var localURL = undefined;
	var	found = false;
	for(var picture in AppMobi.picturelist) {
		if(filename == AppMobi.picturelist[picture]) {
			found=true;
			break;
		}
	}
	if(found)
		localURL = AppMobi.webRoot+'_pictures/'+filename;
	return localURL;
}

AppMobi.addConstructor(function() {
	if (typeof AppMobi.camera == "undefined") AppMobi.camera = new AppMobi.Camera();
});

/**
 * This class provides access to the appMobiAudio
 * @constructor
 */
AppMobi.Audio = function() {
	AppMobiAudio.initRecordingList();
};

AppMobi.Audio.prototype.startPlaying = function(recURL) {
    AppMobiAudio.startPlaying(recURL);
};

AppMobi.Audio.prototype.stopPlaying = function() {
    AppMobiAudio.stopPlaying();
};

AppMobi.Audio.prototype.pausePlaying = function() {
    AppMobiAudio.pausePlaying();
};

AppMobi.Audio.prototype.continuePlaying = function() {
    AppMobiAudio.continuePlaying();
};

AppMobi.Audio.prototype.startRecording = function(format, samplingRate, channels) {
    AppMobiAudio.startRecording(format, samplingRate, channels);
};

AppMobi.Audio.prototype.addSound = function(sound) {
    AppMobiAudio.addSound(sound);
};

AppMobi.Audio.prototype.stopRecording = function() {
    AppMobiAudio.stopRecording();
};

AppMobi.Audio.prototype.pauseRecording = function() {
    AppMobiAudio.pauseRecording();
};

AppMobi.Audio.prototype.continueRecording = function() {
    AppMobiAudio.continueRecording();
};

AppMobi.Audio.prototype.deleteRecording = function(recURL) {
    AppMobiAudio.deleteRecording(recURL);
};

AppMobi.Audio.prototype.clearRecordings = function() {
    AppMobiAudio.clearRecordings();
};

AppMobi.Audio.prototype.getRecordingList = function() {
	var list = [];
	for(var recording in AppMobi.recordinglist) {
		list.push(AppMobi.recordinglist[recording]);
	}
	return list;
}

AppMobi.Audio.prototype.getRecordingURL = function(filename) {
	var localURL = undefined;
	var	found = false;
	for(var recording in AppMobi.recordinglist) {
		if(filename == AppMobi.recordinglist[recording]) {
			found=true;
			break;
		}
	}
	if(found)
		localURL = AppMobi.webRoot+'_recordings/'+filename;
	return localURL;
}

AppMobi.addConstructor(function() {
   if (typeof AppMobi.audio == "undefined") AppMobi.audio = new AppMobi.Audio();
});

/**
 * This class provides access to the appMobiCache
 * @constructor
 */
AppMobi.Cache = function() {
	eval( "AppMobi.cookies = " + AppMobiCache.getCookies());
	eval( "AppMobi.mediacache = " + AppMobiCache.getMediaCache());
};

AppMobi.Cache.prototype.getCookie = function(name) {
	var cookie = undefined;
	try {
		//undelimit single quotes in value of cookie
		cookie = AppMobi.cookies[name].value.replace(/\\'/g, "'");
	} catch(e){};
	return cookie;
};

AppMobi.Cache.prototype.getCookieList = function() {
	var cookies = [];
	for(cookie in AppMobi.cookies) {
		cookies.push(cookie);
	}
	return cookies;
}

//if expires <0, the cookie does not expire
AppMobi.Cache.prototype.setCookie = function(name, value, days) {
	if(name==undefined || name.length==0)
		throw(new Error("Error: AppMobi.cache.setcookie, No cookie name specified."));
	if( name.indexOf('.') > -1 )
		throw(new Error("Error: AppMobi.cache.setcookie, No '.' allowed in cookie names."));
	var tester = "0123456789";
	if( tester.indexOf(name.charAt(0)) > -1 )
		throw(new Error("Error: AppMobi.cache.setcookie, No numbers as first character in cookie names."));

	if(typeof value == "undefined") {
		value = "";
	}

	//make sure value is a string
	value = String(value);

	AppMobi.cookies[name]={"value":value};
	AppMobiCache.setCookie(name, value, days);
};

AppMobi.Cache.prototype.removeCookie = function(name) {
	if(AppMobi.cookies&&AppMobi.cookies[name]) {
		delete AppMobi.cookies[name];
		AppMobiCache.removeCookie(name);
	}
}

AppMobi.Cache.prototype.clearAllCookies = function(name) {
	AppMobi.cookies = {};
	AppMobiCache.clearAllCookies();
}

AppMobi.Cache.prototype.getMediaCacheList = function() {
	var cache = [];
	for(var media in AppMobi.mediacache) {
		cache.push(AppMobi.mediacache[media]);
	}
	return cache;
}

AppMobi.Cache.prototype.getMediaCacheLocalURL = function(url) {
	var localURL = undefined;
	//check if the url is cached
	for(var file in AppMobi.mediacache) {
		if(url==AppMobi.mediacache[file]) {
			localURL = url.split('/');
			localURL = AppMobi.webRoot+'_mediacache/'+localURL[localURL.length-1];
			break;
		}
	}
	return localURL;
}

AppMobi.Cache.prototype.getMediaCacheRelativePath = function(url) {
	var localURL = undefined;
	//check if the url is cached
	for(var file in AppMobi.mediacache) {
		if(url==AppMobi.mediacache[file]) {
			localURL = url.split('/');
			localURL = '_mediacache/'+localURL[localURL.length-1];
			break;
		}
	}
	return localURL;
}

AppMobi.Cache.prototype.clearMediaCache = function() {
	AppMobiCache.clearMediaCache();
};

AppMobi.Cache.prototype.addToMediaCacheImpl = function(url, ID) {
	AppMobiCache.addToMediaCache(url, ID);
};

AppMobi.Cache.prototype.addToMediaCache = function(url) {
	AppMobi.cache.addToMediaCacheImpl(url, "");
};

AppMobi.Cache.prototype.addToMediaCacheExt = function(url, ID) {
	AppMobi.cache.addToMediaCacheImpl(url, ID);
};

AppMobi.Cache.prototype.removeFromMediaCache = function(url) {
	AppMobiCache.removeFromMediaCache(url);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.cache == "undefined") AppMobi.cache = new AppMobi.Cache();
});

/**
 * This class emulates access to the appMobi native GL canvas (which is currently iOS only)
 * @constructor
 */
AppMobi.Canvas = function() {
	this.isHidden = true;
	setTimeout('AppMobi.canvas.messagePump();', 150);
};

AppMobi.Canvas.prototype.load = function(strRelativeFileURL) {
	//reset touch forwarding
    for(eventType in {"touchstart":'',"touchmove":'',"touchend":''}) //TOUCH EVEN FORWARDING CLEANUP
        document.removeEventListener(eventType, AppMobi.canvas.forwardTouchEvent, false);
	AppMobi.canvas.isHidden = false;
	AppMobiCanvas.load(strRelativeFileURL);
};

AppMobi.Canvas.prototype.show = function() {
	AppMobi.canvas.isHidden = false;
	AppMobiCanvas.show();
};

AppMobi.Canvas.prototype.hide = function() {
	AppMobi.canvas.isHidden = true;
	AppMobiCanvas.hide();
};

AppMobi.Canvas.prototype.execute = function(strJavascript) {
	//console.log('AppMobiCanvas.execute:' + strJavascript);
	AppMobiCanvas.execute(strJavascript);
};

AppMobi.Canvas.prototype.eval = function(strJavascript) {
	AppMobiCanvas.eval(strJavascript);
};

AppMobi.Canvas.prototype.reset = function() {
	//reset touch forwarding
    for(eventType in {"touchstart":'',"touchmove":'',"touchend":''}) //TOUCH EVEN FORWARDING CLEANUP
        document.removeEventListener(eventType, AppMobi.canvas.forwardTouchEvent, false);
	AppMobiCanvas.reset();
};

AppMobi.Canvas.prototype.setFramesPerSecond = function(fps) {
	AppMobiCanvas.setFPS(fps);
};

AppMobi.Canvas.prototype.setFPS = function(fps) {
	AppMobiCanvas.setFPS(fps);
};

AppMobi.Canvas.prototype.messagePump = function() {
	try {
		var script = AppMobiCanvas.messagePump();
		if( script ) {
			try { 
				eval(script); 
			} catch(e) { 
				console.log(e.message);
			};
		}
		setTimeout(AppMobi.canvas.messagePump, 15);
	} catch(e) {
		console.log(e);
	}
};

AppMobi.Canvas.prototype.forwardTouchEvent = function(event)
{
    if(this.fwdEvtCounter == undefined) this.fwdEvtCounter = 0;
    //('AppMobi.Canvas.prototype.forwardTouchEvent:'+event.type);
    if(true || event.target.tagName == "HTML")//this check should be here to prevent double-events after bubbling, but it seems like events are not bubbling so with the check we dont get most events
	{
		
    	//build touches, changedTouches, targetTouches strings: JSON arrays of touch data
    	//each touch object has id, x, y
    	var touches = '[';
    	for(var i=0;i<event.touches.length;i++) {
    		
    		var touch = event.touches[i];

    		//dump touch
    		//for(var p in touch){console.log(p+':'+touch[p]);}
    		
    		touches+="[";
    		touches+=touch.identifier+",";
    		touches+=touch.clientX+",";
    		touches+=touch.clientY;
    		touches+="],";
    	}
    	touches+="]";
    	var changedTouches = '[';
    	for(var i=0;i<event.changedTouches.length;i++) {
    		var touch = event.changedTouches[i];
    		changedTouches+="[";
    		changedTouches+=touch.identifier+",";
    		changedTouches+=touch.clientX+",";
    		changedTouches+=touch.clientY;
    		changedTouches+="],";
    	}
    	changedTouches+="]";
    	var targetTouches = '[';
    	for(var i=0;i<event.targetTouches.length;i++) {
    		var touch = event.targetTouches[i];
    		targetTouches+="[";
    		targetTouches+=touch.identifier+",";
    		targetTouches+=touch.clientX+",";
    		targetTouches+=touch.clientY;
    		targetTouches+="],";
    	}
    	targetTouches+="]";
    	
    	
        var sendEvent = false;
        if(event.type == 'touchmove')
        {
            this.fwdEvtCounter++;
            if(this.fwdEvtCounter%3==0){
                sendEvent = true;
                this.fwdEvtCounter=0;
            }    
        }
        else
        {
            sendEvent = true;
        }
		if(sendEvent){
			
			//we expect that screen.width and screen.height are swapped in landscape,
			//but it seems this was changed in Jellybean or is possibly device specific?
			//the following forces h>w in portrait and w>h in landscape
			var screenW = screen.width;
			var screenH = screen.height;
			if(window.orientation%180==0 && screenW>screenH) {
				//portrait and needs swapped
				screenW=screen.height;
				screenH=screen.width;
			} else if(window.orientation%180==90 && screenW<screenH) {
				//landscape and needs swapped
				screenW=screen.height;
				screenH=screen.width;
			}
			
			//var js = 'AppMobi.canvas.setMouseCoords('+window.orientation+','+screen.width+','+screen.height+','+event.target.clientWidth+','+event.target.clientHeight+','+touches+','+targetTouches+','+changedTouches+',"'+event.type.replace('appMobi.','')+'");';
			var js = 'AppMobi.canvas.setMouseCoords('+window.orientation+','+screenW+','+screenH+','+window.innerWidth+','+window.innerHeight+','+touches+','+targetTouches+','+changedTouches+',"'+event.type.replace('appMobi.','')+'");';
			//console.log(js);
            AppMobi.canvas.execute(js);
		}
    }
}

AppMobi.addConstructor(function() {
	if (typeof AppMobi.canvas == "undefined") AppMobi.canvas = new AppMobi.Canvas();
});

/**
 * This class provides access to the appMobi in-app purchasing system
 * @constructor
 */
AppMobi.Purchasing = function() {
};

AppMobi.Purchasing.prototype.purchaseProduct = function(productId, quantity, user) {
	var quan = parseInt(quantity);
	if( typeof(quan) == "undefined" ) {
		q = 1;
	}
	AppMobiPurchasing.purchaseProduct(productId, quan, user);
};

AppMobi.Purchasing.prototype.restorePurchases = function(user) {
	AppMobiPurchasing.restorePurchases(user);
};

AppMobi.addConstructor(function() {
	if (typeof AppMobi.purchasing == "undefined") AppMobi.purchasing = new AppMobi.Purchasing();
});

/**
 * This class provides access to the debugging console.
 * @constructor
 */
AppMobi.Debug = function() {
}

/**
 * Utility function for rendering and indenting strings, or serializing
 * objects to a string capable of being printed to the console.
 * @param {Object|String} message The string or object to convert to an indented string
 * @private
 */
AppMobi.Debug.prototype.processMessage = function(message) {
    if (typeof(message) != 'object') {
        return message;
    } else {
        /**
         * @function
         * @ignore
         */
        function indent(str) {
            return str.replace(/^/mg, "    ");
        }
        /**
         * @function
         * @ignore
         */
        function makeStructured(obj) {
            var str = "";
            for (var i in obj) {
                try {
                    if (typeof(obj[i]) == 'object') {
                        str += i + ":\n" + indent(makeStructured(obj[i])) + "\n";
                    } else {
                        str += i + " = " + indent(String(obj[i])).replace(/^    /, "") + "\n";
                    }
                } catch(e) {
                    str += i + " = EXCEPTION: " + e.message + "\n";
                }
            }
            return str;
        }
        return "Object:\n" + makeStructured(message);
    }
};

/**
 * Print a normal log message to the console
 * @param {Object|String} message Message or object to print to the console
 */
AppMobi.Debug.prototype.log = function(message) {
    if (AppMobi.available)
        AppMobiDebug.log(AppMobi.debug.processMessage(message), { logLevel: 'INFO' });
    else
        console.log(message);
};

/**
 * Print a warning message to the console
 * @param {Object|String} message Message or object to print to the console
 */
AppMobi.Debug.prototype.warn = function(message) {
    if (AppMobi.available)
        AppMobiDebug.log(AppMobi.debug.processMessage(message), { logLevel: 'WARN' });
    else
        console.error(message);
};

/**
 * Print an error message to the console
 * @param {Object|String} message Message or object to print to the console
 */
AppMobi.Debug.prototype.error = function(message) {
    if (AppMobi.available)
        AppMobiDebug.log(AppMobi.debug.processMessage(message), { logLevel: 'ERROR' });
    else
        console.error(message);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.debug == "undefined") AppMobi.debug = new AppMobi.Debug();
});

/**
 * this represents the mobile device, and provides properties for inspecting the model, version, UUID of the
 * phone, etc.
 * @constructor
 */
AppMobi.Device = function() {
    this.available = AppMobi.available;
    this.platform = null;
    this.osversion = null;
    this.model = null;
    this.uuid = null;
    this.initialOrientation = null;
    this.appmobiversion = null;
    this.phonegapversion = null;
    this.cordovaversion = null;
    this.orientation = null;
    this.connection = null;
    this.density = null;
    this.lastPlaying = null;
    this.hasAnalytics = null;
    this.hasCaching = null;
    this.hasStreaming = null;
    this.hasAdvertising = null;
    this.hasPush = null;
    this.hasPayments = null;
    this.hasUpdates = null;
    this.width = null;
    this.height = null;
    this.landwidth = null;
    this.landheight = null;
    this.portwidth = null;
    this.portheight = null;
    this.queryString = null;
    try {
        this.available = true;
        this.platform = AppMobiDevice.getPlatform();
        this.osversion = AppMobiDevice.getOSVersion();
        this.model = AppMobiDevice.getModel();
        this.uuid = AppMobiDevice.getUuid();
        this.initialOrientation = AppMobiDevice.getInitialOrientation();
        this.appmobiversion = AppMobiDevice.getVersion();
        this.phonegapversion = AppMobiDevice.getPGVersion();
        try {
		this.cordovaversion = AppMobiDevice.getCordovaVersion();
	} catch(e) {}
        this.orientation = AppMobiDevice.getOrientation();
        this.connection = AppMobiDevice.getConnection();
        this.density = AppMobiDevice.getDisplayDensity();
        this.lastPlaying = AppMobiDevice.getPlayingStation();
        this.hasAnalytics = AppMobiDevice.getHasAnalytics();
        this.hasCaching = AppMobiDevice.getHasCaching();
        this.hasStreaming = AppMobiDevice.getHasStreaming();
        this.hasAdvertising = AppMobiDevice.getHasAdvertising();
	this.hasPush = AppMobiDevice.getHasPush();
	this.hasPayments = AppMobiDevice.getHasPayments();
	this.hasUpdates = AppMobiDevice.getHasUpdates();
	this.queryString = AppMobiDevice.getQueryString();
        this.width = AppMobiDevice.getDisplayWidth();
        this.height = AppMobiDevice.getDisplayHeight();
        this.landwidth = AppMobiDevice.getLandWidth();
        this.landheight = AppMobiDevice.getLandHeight();
        this.portwidth = AppMobiDevice.getPortWidth();
        this.portheight = AppMobiDevice.getPortHeight();
    } catch(e) {
    }
    
    //setup polling for keyboard visibility
    this.isSoftKeyboardShowing = false;
    this.checkKeyboard = function() {
    	if(this.isSoftKeyboardShowing!=AppMobiDevice.isSoftKeyboardShowing()) {
    		this.isSoftKeyboardShowing=AppMobiDevice.isSoftKeyboardShowing();
    		var e = document.createEvent('Events');
    		e.initEvent((this.isSoftKeyboardShowing?'appMobi.device.keyboard.show':'appMobi.device.keyboard.hide'),true,true);
    		document.dispatchEvent(e);
    	}
    	setTimeout("AppMobi.device.checkKeyboard()", 250);
    };
    this.checkKeyboard();
}

/**
 * This class specifies the attributes for getRemoteDataExt.
 * @constructor
 */
AppMobi.Device.RemoteDataParameters = function() {
	this.url = "";
	this.id = "";
	this.method = "GET";
	this.body = "";
	this.headers = "";
};

AppMobi.Device.RemoteDataParameters.prototype.addHeader = function(name, value) {
	if(typeof name != "string" || name.length==0) {
		throw(new Error("Error: the name parameter must be of type 'string' and must be of length > 0 for RemoteDataParameters"));
	}
	
	if(typeof value != "string") {		
		if(typeof value == "undefined") {
			//convert undefined to zero-length string
			value = "";
		} else {
			//convert other non-strings (function, number, object, etc.) to string representation
			value = String(value);
		}		
	}
	
	this.headers += name.length +"~" + name + value.length +"~" + value;
};

AppMobi.Device.prototype.managePower = function(shouldStayOn, onlyWhenPluggedIn) {
	AppMobiDevice.managePower(shouldStayOn, onlyWhenPluggedIn);
};

AppMobi.Device.prototype.setAutoRotate = function(shouldAutoRotate) {
	AppMobiDevice.setAutoRotate(shouldAutoRotate);
};

AppMobi.Device.prototype.setRotateOrientation = function(orientation) {
	//orientation should be 'portrait' or 'landscape'
	AppMobiDevice.setRotateOrientation(orientation);
};

AppMobi.Device.prototype.updateConnection = function() {
	AppMobiDevice.updateConnection();
};

AppMobi.Device.prototype.setBasicAuthentication = function(host, username, password) {
	AppMobiDevice.setBasicAuthentication(host, username, password);
};

AppMobi.Device.prototype.addVirtualPage = function() {
	AppMobiDevice.addVirtualPage();
};

AppMobi.Device.prototype.removeVirtualPage = function() {
	AppMobiDevice.removeVirtualPage();
};

AppMobi.Device.prototype.addMenuItem = function(text, callback) {
	AppMobiDevice.addMenuItem(text, callback);
};

AppMobi.Device.prototype.registerLibrary = function(strDelegateName) {
	AppMobiDevice.registerLibrary(strDelegateName);
};

AppMobi.Device.prototype.launchExternal = function(strURL) {
	AppMobiDevice.launchExternal(strURL);
};

AppMobi.Device.prototype.showRemoteSite = function(strURL, closeX, closeY, closeWidth, closeHeight) {
	AppMobi.device.showRemoteSiteExt(strURL, closeX, closeY, closeX, closeY, closeWidth, closeHeight);
};

AppMobi.Device.prototype.showRemoteSiteExt = function(strURL, closePortX, closePortY, closeLandX, closeLandY, closeWidth, closeHeight, closeImage) {
	if(closeImage == undefined) closeImage = null;
	
	var new_fn = true;
    try { AppMobiDevice.showRemoteSite(strURL, closePortX, closePortY, closeLandX, closeLandY, closeWidth, closeHeight, closeImage); } catch(e) { new_fn = false; }
    if( new_fn == false )
    {
    	new_fn = true;
    	try { AppMobiDevice.showRemoteSite(strURL, closePortX, closePortY, closeLandX, closeLandY, closeWidth, closeHeight); } catch(e) { new_fn = false; }
    }
    if( new_fn == false )
    {
    	AppMobiDevice.showRemoteSite(strURL, closePortX, closePortY, closeWidth, closeHeight);
    }
};

AppMobi.Device.prototype.closeRemoteSite = function() {
	AppMobiDevice.closeRemoteSite();
};

AppMobi.Device.prototype.mainViewExecute = function(js) {
	AppMobiDevice.mainViewExecute(js);
};

//internal only
AppMobi.adBlockBeforeUnloadListener = function(e) {
	AppMobiDevice.stopLoading();
}

AppMobi.Device.prototype.blockRemotePages = function(shouldblock, whitelist) {
	try {
		if(shouldblock) {
			AppMobiDevice.setBlockedPagesWhitelist(whitelist);
			window.addEventListener("beforeunload", AppMobi.adBlockBeforeUnloadListener, true);
		} else {
			window.removeEventListener("beforeunload", AppMobi.adBlockBeforeUnloadListener, true);
		}
	}
	catch(e) {
		console.log(e);
	}
};

AppMobi.Device.prototype.scanBarcode = function() {
	AppMobiDevice.scanBarcode();
};

AppMobi.Device.prototype.rateApplication = function(appid) {
    if(typeof appid != "string" || appid==0) {
		throw(new Error("Error: AppMobi.device.rateApplication: appname must be a non empty string."));
	}
	AppMobiDevice.rateApplication(appid);
};

AppMobi.Device.prototype.runInstallNativeApp = function(appname, runcommand, installurl, bundleid) {
    if(typeof appname != "string" || appname.length==0 ||
       typeof runcommand != "string" || runcommand.length==0 ||
       typeof installurl != "string" || installurl.length==0 ||
       typeof bundleid != "string" || bundleid==0) {
		throw(new Error("Error: AppMobi.device.runInstallNativeApp: appname, runcommand,installurl and bundleid must be non empty strings."));
	}
	AppMobiDevice.runInstallNativeApp(appname, runcommand, installurl, bundleid);
};

AppMobi.Device.prototype.sendEmail = function(body, to, subject, ishtml, cc, bcc) {
    if(typeof body != "string" || body.length==0) {
		throw(new Error("Error: AppMobi.device.sendEmail: body must be a non empty string."));
	}
    if(typeof to != "string" || to.length==0 ) to = "";
    if(typeof subject != "string" || subject.length==0 ) subject = "";
    if(typeof ishtml != "boolean" ) ishtml = false;
    if(typeof cc != "string" || cc.length==0 ) cc = "";
    if(typeof bcc != "string" || bcc.length==0 ) bcc = "";
	AppMobiDevice.sendEmail(body, to, subject, ishtml, cc, bcc);
};

AppMobi.Device.prototype.sendSMS = function(body, to) {
    if(typeof body != "string" || body.length==0) {
		throw(new Error("Error: AppMobi.device.sendSMS: body must be a non empty string."));
	}
    if(typeof to != "string" || to.length==0 ) to = "";
	AppMobiDevice.sendSMS(body, to);
};

AppMobi.Device.prototype.hideStatusBar = function() {
	AppMobiDevice.hideStatusBar();
};


/**
 * Set the current orientation of the phone.  This is called from the device automatically.
 *
 * When the orientation is changed, the DOMEvent \c orientationChanged is dispatched against
 * the document element.  The event has the property \c orieentation which can be used to retrieve
 * the device's current orientation.
 *
 * @param {Number} orientation The orientation to be set
 */
AppMobi.Device.prototype.setOrientation = function(orientation) {
	AppMobi.device.orientation = orientation;

	var e = document.createEvent('Events');
	e.initEvent('appMobi.device.orientation.change', true, true);
	e.orientation = orientation;
	document.dispatchEvent(e);
};

AppMobi.Device.prototype.getRemoteDataImpl = function(requestUrl, requestMethod, requestBody, successCallback, errorCallback, id, hasId) {
	//validate parameters
	if(
		 (requestUrl == undefined || requestUrl=='') ||
		 (requestMethod == undefined || requestMethod=='' || (requestMethod.toUpperCase()!='GET' && requestMethod.toUpperCase()!='POST')) ||
		 (successCallback == undefined || successCallback=='') ||
		 (errorCallback == undefined || errorCallback=='')
		 ) {
		throw(new Error("Error: AppMobi.device.getRemoteData has the following required parameters: requestUrl, requestMethod, requestBody, successCallback, errorCallback.  requestMethod must be either GET or POST.  requestBody is ignored for GET requests."));
	}

	if(typeof(successCallback)!="string") throw(new Error("Error: AppMobi.device.getRemoteData successCallback parameter must be a string which is the name of a function"));
	if(typeof(errorCallback)!="string") throw(new Error("Error: AppMobi.device.getRemoteData errorCallback parameter must be a string which is the name of a function"));

	if(requestBody == undefined) requestBody="";
	if( requestUrl.indexOf('http') != 0 ) requestUrl = AppMobi.webRoot + requestUrl;
	
	AppMobiDevice.getRemoteData(requestUrl, requestMethod, requestBody, successCallback, errorCallback, id, hasId);
};

AppMobi.Device.prototype.getRemoteDataWithId = function(requestUrl, requestMethod, requestBody, successCallback, errorCallback, id) {
	AppMobi.device.getRemoteDataImpl(requestUrl, requestMethod, requestBody, successCallback, errorCallback, id, true);
};

AppMobi.Device.prototype.getRemoteDataWithID = function(requestUrl, requestMethod, requestBody, successCallback, errorCallback, id) {
	AppMobi.device.getRemoteDataImpl(requestUrl, requestMethod, requestBody, successCallback, errorCallback, id, true);
};

AppMobi.Device.prototype.getRemoteData = function(requestUrl, requestMethod, requestBody, successCallback, errorCallback) {
	AppMobi.device.getRemoteDataImpl(requestUrl, requestMethod, requestBody, successCallback, errorCallback, "", false);
};

AppMobi.Device.prototype.getRemoteDataExt = function(parameters) {
	if( parameters == undefined )
	{
		throw(new Error("Error: AppMobi.device.getRemoteDataExt: parameters is required."));
	}

	if( parameters.hasOwnProperty("url") == false || parameters.hasOwnProperty("id") == false || parameters.hasOwnProperty("method") == false 
	   || parameters.hasOwnProperty("body") == false || parameters.hasOwnProperty("headers") == false )
	{
		throw(new Error("Error: AppMobi.device.getRemoteDataExt: invalid parameters object. Initialize using 'new AppMobi.Device.RemoteDataParameters'."));
	}
	
	if( parameters.url == undefined || parameters.url=='' )
	{
		throw(new Error("Error: AppMobi.device.getRemoteDataExt requires a url property."));
	}

	if( parameters.method == undefined || (parameters.method.toUpperCase()!='GET' && parameters.method.toUpperCase()!='POST') )
	{
		throw(new Error("Error: AppMobi.device.getRemoteDataExt requires a method property of GET or POST. body is ignored for GET requests."));
	}
	
	if( parameters.url.indexOf('http') != 0 ) parameters.url = AppMobi.webRoot + parameters.url;
	AppMobiDevice.getRemoteDataExt(parameters.url, parameters.id, parameters.method, parameters.body, parameters.headers);
};

AppMobi.Device.prototype.installUpdate = function() {
	AppMobiDevice.installUpdate();
};

AppMobi.Device.prototype.hideSplashScreen = function() {
	AppMobiDevice.hideSplashScreen();
};

AppMobi.Device.prototype.cachePackage = function(subdir, url) {
	if( typeof(subdir) != 'string' || typeof(url) != 'string' || subdir.length == 0 || url.length == 0 )
	{
		throw(new Error("Error: AppMobi.device.cachePackage: subdir and url are required."));
	}

	AppMobiDevice.cachePackage(subdir, url);
};

AppMobi.Device.prototype.messagePump = function()
{
	if(typeof(AppMobiDevice.messagePump) == "function")
	{
		var script = AppMobiDevice.messagePump();
		if( script ) {
			try { eval(script); } catch(e) { console.log(e); };
		}
		setTimeout('AppMobi.device.messagePump();', 15);
	}
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.device == "undefined") AppMobi.device = new AppMobi.Device();
});

/**
 * This class provides access to the appMobiGeolocation
 * @constructor
 */
AppMobi.Geolocation = function() {
	this.watchIDs = new Array();
	this.successCBs = new Array();
	this.errorCBs = new Array();
	this.pollID = -1;
};

AppMobi.Geolocation.Coords = function(latitude, longitude, altitude, accuracy, altitudeAccuracy, heading, speed) {
	this.latitude = latitude;
	this.longitude = longitude;
	this.altitude = altitude;
	this.accuracy = accuracy;
	this.altitudeAccuracy = altitudeAccuracy;
	this.heading = heading;
	this.speed = speed;
};

AppMobi.Geolocation.Position = function(coords, timestamp) {
	this.coords = coords;
	this.timestamp = timestamp;
};

AppMobi.Geolocation.prototype.poll = function() {
	var i, to = 0, len = this.watchIDs.length;
	//AppMobiGeolocation.printMessage("poll: watchIDs.length = " + len);
	for (i = 0; i < len; ++i) {
		var loc = AppMobiGeolocation.pollLocation(this.watchIDs[i]);
		if (loc.length()) {	
			loc = new String(loc);	// Convert to actual string.
			var vals = loc.split(",");
			var once = vals[0];
			//AppMobiGeolocation.printMessage("poll: vals.length = " + vals.length + ", once = " + once + ", vals = " + vals);
			this.successCB(vals[1],vals[2],vals[3],vals[4],vals[5],vals[6],vals[7],vals[8], vals[9]);
			if (once == 0)
				this.watchIDs[to++] = this.watchIDs[i];
		} else
			this.watchIDs[to++] = this.watchIDs[i];
	}
	if (to < len)
		this.watchIDs.splice(to);
};

AppMobi.Geolocation.prototype.getSuccessId = function(successCallback) {
	var i, len = this.successCBs.length;
	for (i = 0; i < len; ++i) {
		if (this.successCBs[i] == successCallback)
			return i;
	}
	len = this.successCBs.push(successCallback);
	return len - 1;
};

AppMobi.Geolocation.prototype.getErrorId = function(errorCallback) {
	var i, len = this.errorCBs.length;
	for (i = 0; i < len; ++i) {
		if (this.errorCBs[i] == errorCallback)
			return i;
	}
	len = this.errorCBs.push(errorCallback);
	return len - 1;
};

AppMobi.Geolocation.prototype.successCB = function(ID, latitude, longitude, altitude, accuracy,
			altitudeAccuracy, heading, speed, timestamp) {
	var fun = this.successCBs[ID];
	var coords = new AppMobi.Geolocation.Coords(latitude, longitude, altitude, accuracy, altitudeAccuracy, heading, speed);
	var p = new AppMobi.Geolocation.Position(coords, timestamp);
	fun(p);
};

AppMobi.Geolocation.prototype.errorCB = function(ID) {
	var fun = this.errorCBs[ID];
	fun();
};

AppMobi.Geolocation.prototype.getCurrentPosition = function(successCallback, errorCallback, options) {
	if( (successCallback == undefined || successCallback==''))
	{
		throw(new Error("Error: getCurrentPosition has the following required parameters: successCallback."));
	}
	var successID = this.getSuccessId(successCallback);
	var errorID = errorCallback == undefined ? -1 : this.getErrorId(errorCallback);
	var enableHighAccuracy = true;
	var maximumAge = -1;
	if (options != undefined) {
		if (options.maximumAge != undefined)
			maximumAge = options.maximumAge;
		if (options.enableHighAccuracy != undefined)
			enableHighAccuracy = options.enableHighAccuracy;
	}
	var id = AppMobiGeolocation.getCurrentPosition(successID, errorID, maximumAge, enableHighAccuracy);
	//AppMobiGeolocation.printMessage("getCurrentPosition: " + id + ", pollID = " + this.pollID);
	this.watchIDs.push(id);
	if (this.pollID == -1)
		this.pollID = setInterval("AppMobi.geolocation.poll()", 200);	// Poll every 200 msecs.
};

AppMobi.Geolocation.prototype.watchPosition = function(successCallback, errorCallback, options) {
	if( (successCallback == undefined || successCallback==''))
	{
		throw(new Error("Error: watchPosition has the following required parameters: successCallback."));
	}
	var successID = this.getSuccessId(successCallback);
	var errorID = errorCallback == undefined ? -1 : this.getErrorId(errorCallback);
	var freq = 10000;
	var enableHighAccuracy = true;
	var maximumAge = -1;
	if (options != undefined) {
		if (options.timeout != undefined)
			freq = options.timeout;
		if (options.maximumAge != undefined)
			maximumAge = options.maximumAge;
		if (options.enableHighAccuracy != undefined)
		enableHighAccuracy = options.enableHighAccuracy;
	}
	var id = AppMobiGeolocation.watchPosition(successID, errorID, freq, maximumAge, enableHighAccuracy);
	//AppMobiGeolocation.printMessage("watchPosition: " + id + ", pollID = " + this.pollID);
	this.watchIDs.push(id);
	if (this.pollID == -1) {
		//AppMobiGeolocation.printMessage("About to schedule poll()");
		this.pollID = setInterval("AppMobi.geolocation.poll()", 200);	// Poll every 200 msecs.
	}
	return id;
};

AppMobi.Geolocation.prototype.clearWatch = function(id) {
	AppMobiGeolocation.clearWatch(id);
	var index = this.watchIDs.indexOf(id);
	if (index != -1) {
		this.watchIds.splice(index, 1);
		if (this.watchIds.length == 0 && this.pollID != -1) {
			clearInterval(this.pollID);
			this.pollID = -1;
		}
	}
};

// Use OLD method until we finish final testing on new object
//AppMobi.addConstructor(function() {
//	if (typeof AppMobi.geolocation == "undefined") AppMobi.geolocation = new AppMobi.Geolocation();
//});

/* ++++++THE OLD METHOD.  To be replaced by AppMobi.Geolocation.*/
AppMobi.addConstructor(function() {
    if (typeof AppMobi.geolocation == "undefined") AppMobi.geolocation = navigator.geolocation;
});

/**
 * This class provides access to notifications on the device.
 */
AppMobi.Notification = function() {
	eval(AppMobiNotification.getNotificationsString());
}

/**
 * This class specifies the attributes for push users.
 * @constructor
 */
AppMobi.Notification.PushUserAttributes = function() {
	this.s1 = null;
	this.s2 = null;
	this.s3 = null;
	this.s4 = null;
	this.s5 = null;
	this.s6 = null;
	this.n1 = null;
	this.n2 = null;
	this.n3 = null;
	this.n4 = null;
};

/**
 * Open a native alert dialog, with a customizable title and button text.
 * @param {String} message Message to print in the body of the alert
 * @param {String} [title="Alert"] Title of the alert dialog (default: Alert)
 * @param {String} [buttonLabel="OK"] Label of the close button (default: OK)
 */
AppMobi.Notification.prototype.alert = function(message, title, button) {
    if (AppMobi.available)
        AppMobiNotification.alert(message, title, button);
    else
        alert(message);
};

AppMobi.Notification.prototype.confirm = function(message, id, title, ok, cancel) {
    AppMobiNotification.confirm(message, id, title, ok, cancel);
};

/**
 * Causes the device to vibrate.
 */
AppMobi.Notification.prototype.vibrate = function() {
    AppMobiNotification.vibrate();
};

/**
 * Causes the device to beep.
 * @param {Integer} count The number of beeps.
 */
AppMobi.Notification.prototype.beep = function(count) {
    AppMobiNotification.beep(count);
};

AppMobi.Notification.prototype.showBusyIndicator = function() {
    AppMobiNotification.showBusyIndicator();
};

AppMobi.Notification.prototype.hideBusyIndicator = function() {
    AppMobiNotification.hideBusyIndicator();
};

AppMobi.Notification.prototype.getNotificationList = function() {
	var notify = [];
	for(var note in AppMobi.notifications) {
		notify.push(AppMobi.notifications[note].id);
	}
	return notify;
};

AppMobi.Notification.prototype.getNotificationData = function(id) {
	var local = null;
	for(var note in AppMobi.notifications) {
		if(id==AppMobi.notifications[note].id) {
			local = {};
			local.id = AppMobi.notifications[note].id;
			local.msg = AppMobi.notifications[note].msg;
			local.data = AppMobi.notifications[note].data;
			local.userkey = AppMobi.notifications[note].userkey;
			local.richurl = AppMobi.notifications[note].richurl;
			local.richhtml = AppMobi.notifications[note].richhtml;
			local.isRich = AppMobi.notifications[note].isRich;
			break;
		}
	}
	return local;
};

AppMobi.Notification.prototype.checkPushUser = function(userID, password) {
	if( userID   == undefined || userID   == "" ||
		 password == undefined || password == "" )
	{
		throw(new Error("Error: AppMobi.notification.checkPushUser, No user or password specified."));
	}
	
	AppMobiNotification.checkPushUser(userID, password);
};

AppMobi.Notification.prototype.addPushUser = function(userID, password, email) {
	if( userID   == undefined || userID   == "" || userID.indexOf(' ')!=-1 || userID.indexOf('.')!=-1 || 
		password == undefined || password == "" || password.indexOf(' ')!=-1 || password.indexOf('.')!=-1 ||
		email    == undefined || email    == "" )
	{
		throw(new Error("Error: AppMobi.notification.addPushUser: User, email and password are required. The space (' ') and dot ('.') characters are not allowed in user and password."));
	}

	AppMobiNotification.addPushUser(userID, password, email);
};

AppMobi.Notification.prototype.editPushUser = function(newEmail, newPassword, newUser) {
	if( ( newEmail == undefined || newEmail == "" ) &&
        ( newPassword == undefined || newPassword == "" || newPassword.indexOf(' ')!=-1 || newPassword.indexOf('.')!=-1 ) && 
        ( newUser == undefined || newUser == "" || newUser.indexOf(' ')!=-1 || newUser.indexOf('.')!=-1 ) )
	{
		throw(new Error("Error: AppMobi.notification.editPushUser, No new value (email or password or user) specified. The space (' ') and dot ('.') characters are not allowed in user or password."));
	}

	AppMobiNotification.editPushUser(newEmail, newPassword, newUser);
};

AppMobi.Notification.prototype.deletePushUser = function() {
	AppMobiNotification.deletePushUser();
};

AppMobi.Notification.prototype.sendPushUserPass = function() {
	AppMobiNotification.sendPushUserPass();
};

AppMobi.Notification.prototype.setPushUserAttributes = function(attributes) {
	if( attributes == undefined )
	{
		throw(new Error("Error: AppMobi.notification.setPushUserAttributes: attributes is required."));
	}

	if( attributes.hasOwnProperty("s1") == false || attributes.hasOwnProperty("s2") == false || attributes.hasOwnProperty("s3") == false || attributes.hasOwnProperty("s4") == false 
	 || attributes.hasOwnProperty("s5") == false || attributes.hasOwnProperty("s6") == false || attributes.hasOwnProperty("n1") == false
	 || attributes.hasOwnProperty("n2") == false || attributes.hasOwnProperty("n3") == false || attributes.hasOwnProperty("n4") == false )
	{
		throw(new Error("Error: AppMobi.notification.setPushUserAttributes: invalid attributes parameter specified. Initialize using 'new AppMobi.Notification.PushUserAttributes'."));
	}

	if( (Number(attributes.n1) == NaN) || (Number(attributes.n2) == NaN) || 
			(Number(attributes.n3) == NaN) || (Number(attributes.n4) == NaN) )
	{
		throw(new Error("Error: AppMobi.notification.setPushUserAttributes: attributes n1,n2,n3,n4 must be numbers."));
	}

	var parsedAttributes = "";
	for(var prop in attributes) {
		if( (prop=="s1"||prop=="s2"||prop=="s3"||prop=="s4"||prop=="s5"||prop=="s6"||prop=="n1"||prop=="n2"||prop=="n3"||prop=="n4") && attributes[prop] != null ) {
			parsedAttributes += "[";
			parsedAttributes += prop;
			parsedAttributes += "=";
			parsedAttributes += escape(attributes[prop]);
			parsedAttributes += "]";
		}
	}

	AppMobiNotification.setPushUserAttributes(parsedAttributes);
};

AppMobi.Notification.prototype.findPushUser = function(userID, email) {
	AppMobiNotification.findPushUser(userID, email);
};

AppMobi.Notification.prototype.refreshPushNotifications = function() {
	AppMobiNotification.refreshPushNotifications();
};

AppMobi.Notification.prototype.refreshUserPushNotifications = function(user, pass, device, newerthan) {
	AppMobiNotification.refreshUserPushNotifications(user, pass, device, newerthan);
};

AppMobi.Notification.prototype.readPushNotifications = function(notificationIDs) {
	if( notificationIDs == undefined || notificationIDs == "")
	{
		throw(new Error("Error: AppMobi.notification.readPushNotifications, No notificationIDs specified."));
	}
	
	if( typeof( notificationIDs ) == "number" ) notificationIDs = "" + notificationIDs;

	AppMobiNotification.readPushNotifications(notificationIDs);
};

AppMobi.Notification.prototype.deletePushNotifications = function(notificationIDs) {
	if( notificationIDs == undefined || notificationIDs == "")
	{
		throw(new Error("Error: AppMobi.notification.deletePushNotifications, No notificationIDs specified."));
	}
	
	if( typeof( notificationIDs ) == "number" ) notificationIDs = "" + notificationIDs;

	AppMobiNotification.readPushNotifications(notificationIDs);
};

AppMobi.Notification.prototype.sendPushNotification = function(userID, message, data) {
	if( userID  == undefined || userID  == "" ||
		message == undefined || message == "" )
	{
		throw(new Error("Error: AppMobi.notification.sendPushNotification, No user or message specified."));
	}
	
    if( typeof( data ) != "string" ) data = "";	
	if( message.length > 160 ) throw(new Error("Error: AppMobi.notification.sendPushNotification, message cannot exceed 160 characters in length."));
	if( data.length > 1024 ) throw(new Error("Error: AppMobi.notification.sendPushNotification, data cannot exceed 1024 characters in length."));

	AppMobiNotification.sendPushNotification(userID, message, data);
};

AppMobi.Notification.prototype.broadcastPushNotification = function(message, data, attributes, skip) {
	if( message == undefined || message == "" )
	{
		throw(new Error("Error: AppMobi.notification.broadcastPushNotification, No message specified."));
	}
    
    if( typeof( data ) != "string" ) data = "";	
	if( message.length > 1024 ) throw(new Error("Error: AppMobi.notification.broadcastPushNotification, message cannot exceed 1024 characters in length."));
	if( data.length > 1024 ) throw(new Error("Error: AppMobi.notification.broadcastPushNotification, data cannot exceed 1024 characters in length."));
    
	AppMobiNotification.broadcastPushNotification( message, data, attributes, skip);
};

AppMobi.Notification.prototype.showRichPushViewer = function(notificationID, closePortX, closePortY, closeLandX, closeLandY, closeWidth, closeHeight) {
	if( notificationID == undefined || notificationID  == "")
	{
		throw(new Error("Error: AppMobi.notification.showRichPushViewer, No notification ID specified."));
	}
	
	var notification = AppMobi.notification.getNotificationData( notificationID );
	if( notification == null || ( notification.richurl == "" && notification.richhtml == "" ) )
	{
		throw(new Error("Error: AppMobi.notification.showRichPushViewer, This notification ID is not a rich message."));
	}
	
	AppMobiNotification.showRichPushViewer(notificationID, closePortX, closePortY, closeLandX, closeLandY, closeWidth, closeHeight);
};

AppMobi.Notification.prototype.closeRichPushViewer = function() {
	AppMobiNotification.closeRichPushViewer();
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.notification == "undefined") AppMobi.notification = new AppMobi.Notification();
});

/**
 * This class provides access to the device display.
 * @constructor
 */
AppMobi.Display = function() {
    this.viewport = {};
    this.oldviewport = {};
    this.window = {};
    this.window.landwidth = 0;
    this.window.portwidth = 0;
    this.window.landheight = 0;
    this.window.portheight = 0;
}

/**
//turns on appmobi managed viewport
 * @param {int} portraitWidthInPx
 * @param {int} landscapeWidthInPx
 */
AppMobi.Display.prototype.useViewport = function(portraitWidthInPx, landscapeWidthInPx) {
    AppMobi.display.viewport.portraitWidth = parseInt(portraitWidthInPx);
    AppMobi.display.viewport.landscapeWidth = parseInt(landscapeWidthInPx);
    if(isNaN(AppMobi.display.viewport.portraitWidth)||isNaN(AppMobi.display.viewport.landscapeWidth)) return;
    document.addEventListener('appMobi.device.orientation.change', AppMobi.display.viewportOrientationListener, false);
    AppMobi.display.updateViewportOrientation(AppMobi.device.orientation);
}

AppMobi.Display.prototype.updateViewportContent = function(content) {
    //get reference to head
    var head, heads = document.getElementsByTagName('head');
    if(heads.length>0) head = heads[0];
    else return;
    //remove any viewport meta tags
    var metas = document.getElementsByTagName('meta');
    for(var i=0;i<metas.length;i++) {
        if(metas[i].name=='viewport') try {head.removeChild(metas[i]);} catch(e){}
    }
    //add the new viewport meta tag
    var viewport = document.createElement('meta');
    viewport.setAttribute('name', 'viewport');
    viewport.setAttribute('id', 'viewport');
    viewport.setAttribute('content', content);
    head.appendChild(viewport);
}

AppMobi.Display.prototype.updateViewportOrientation = function(orientation) {
    var width, deviceWidth;
    if(orientation==0||orientation==180) {
        width=AppMobi.display.viewport.portraitWidth;
    } else {
        width=AppMobi.display.viewport.landscapeWidth;
    }
    deviceWidth = AppMobi.device.width;
    var scale = (deviceWidth)/width;
	
	if(AppMobi.hasOwnProperty('compatibilityModeDisabled')) {
	    //adjust scale based on density if compatibility mode is disabled
	    scale = scale/AppMobiDevice.getDisplayDensity();
	}
		
    //round scale to nearest hundredth
    scale = (Math.round(scale*100))/100;

    if(AppMobiDevice.hasHTCUndocumentedMethods() && AppMobiDevice.isActive()) {
    	AppMobi.display.updateViewportContent('minimum-scale='+scale+',maximum-scale='+scale);//if any-density=true, prepend with target-densitydpi=device-dpi,
	    AppMobiDisplay.forceScale(scale);
	    scrollTo(0,0);
    } else {
    	AppMobi.display.updateViewportContent('minimum-scale='+scale+',maximum-scale='+scale);//if any-density=true, prepend with target-densitydpi=device-dpi,
    }
    //var osver = parseFloat(AppMobi.device.osversion);
    //AppMobi.debug.log("AppMobi.device.orientation:"+AppMobi.device.orientation+",orientation:"+orientation+",width:"+width+",deviceWidth:"+deviceWidth+",screen.width:"+screen.width+",AppMobi.device.density:"+AppMobi.device.density+",osver:"+osver+",scale:"+scale);
    //AppMobi.debug.log("AppMobi.display.viewport.portraitWidth:"+AppMobi.display.viewport.portraitWidth+",AppMobi.display.viewport.landscapeWidth:"+AppMobi.display.viewport.landscapeWidth);
	//AppMobi.debug.log(location.href);
}

AppMobi.Display.prototype.viewportOrientationListener = function(e){
    AppMobi.display.updateViewportOrientation(AppMobi.device.orientation);
}

AppMobi.Display.prototype.startAR = function() {
    AppMobiDisplay.startAR();
};

AppMobi.Display.prototype.stopAR = function() {
    AppMobiDisplay.stopAR();
};

AppMobi.Display.prototype.switchViewport = function(currentDoc, portraitWidthInPx, landscapeWidthInPx) {
	this.oldviewport.portraitWidth = this.viewport.portraitWidth;
	this.oldviewport.landscapeWidth = this.viewport.landscapeWidth;
	this.useViewport(portraitWidthInPx, landscapeWidthInPx);
};

AppMobi.Display.prototype.revertViewport = function() {
	this.useViewport(this.oldviewport.portraitWidth, this.oldviewport.landscapeWidth);
};

AppMobi.Display.prototype.lockViewportWindow = function (portwidth, portheight, landwidth, landheight)
{
    var vplandwidth = parseInt( ( parseFloat(landheight) / parseFloat(AppMobi.device.landheight) ) * parseFloat(AppMobi.device.landwidth) );
    var vpportwidth = parseInt( ( parseFloat(portheight) / parseFloat(AppMobi.device.portheight) ) * parseFloat(AppMobi.device.portwidth) );
    var vplandheight = parseInt( ( parseFloat(landwidth) / parseFloat(AppMobi.device.landwidth) ) * parseFloat(AppMobi.device.landheight) );
    var vpportheight = parseInt( ( parseFloat(portwidth) / parseFloat(AppMobi.device.portwidth) ) * parseFloat(AppMobi.device.portheight) );
    
    if( vpportwidth > portwidth )
    {
    	AppMobi.display.window.portwidth = vpportwidth;
    	AppMobi.display.window.portheight = portheight 
    }
    else
    {
    	AppMobi.display.window.portwidth = portwidth;
    	AppMobi.display.window.portheight = vpportheight;
    }
    
    if( vplandwidth > landwidth )
    {
    	AppMobi.display.window.landwidth = vplandwidth;
    	AppMobi.display.window.landheight = landheight
    }
    else
    {
    	AppMobi.display.window.landwidth = landwidth;
    	AppMobi.display.window.landheight = vplandheight;
    }
    AppMobi.display.useViewport( AppMobi.display.window.portwidth, AppMobi.display.window.landwidth );
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.display == "undefined") AppMobi.display = new AppMobi.Display();
});

/**
 * This class provides access to oauth.
 * @constructor
 */
AppMobi.OAuth = function() {
}

/**
 * This class specifies the attributes for getRemoteDataExt.
 * @constructor
 */
AppMobi.OAuth.ProtectedDataParameters = function() {
	this.service = "";
	this.url = "";
	this.id = "";
	this.method = "GET";
	this.body = "";
	this.headers = "";
};

AppMobi.OAuth.ProtectedDataParameters.prototype.addHeader = function(name, value) {
	if(typeof name != "string" || name.length==0) {
		throw(new Error("Error: the name parameter must be of type 'string' and must be of length > 0 for ProtectedDataParameters"));
	}
	
	if(typeof value != "string") {		
		if(typeof value == "undefined") {
			//convert undefined to zero-length string
			value = "";
		} else {
			//convert other non-strings (function, number, object, etc.) to string representation
			value = String(value);
		}		
	}
	
	this.headers += name.length +"~" + name + value.length +"~" + value;
};

AppMobi.OAuth.prototype.unauthorizeService = function(service) {
	AppMobiOAuth.unauthorizeService(service);
}

AppMobi.OAuth.prototype.getProtectedData = function(parameters) {
	if( parameters == undefined )
	{
		throw(new Error("Error: AppMobi.oauth.getProtectedData: parameters is required."));
	}
	
	if( parameters.hasOwnProperty("service") == false || parameters.hasOwnProperty("url") == false || parameters.hasOwnProperty("id") == false 
	    || parameters.hasOwnProperty("method") == false || parameters.hasOwnProperty("body") == false || parameters.hasOwnProperty("headers") == false )
	{
		throw(new Error("Error: AppMobi.oauth.getProtectedData: invalid parameters object. Initialize using 'new AppMobi.Device.RemoteDataParameters'."));
	}
	
	if( parameters.service == undefined || parameters.service=='' )
	{
		throw(new Error("Error: AppMobi.oauth.getProtectedData requires a service property."));
	}
	
	if( parameters.url == undefined || parameters.url=='' )
	{
		throw(new Error("Error: AppMobi.oauth.getProtectedData requires a url property."));
	}
	
	if( parameters.method == undefined || (parameters.method.toUpperCase()!='GET' && parameters.method.toUpperCase()!='POST') )
	{
		throw(new Error("Error: AppMobi.oauth.getProtectedData requires a method property of GET or POST. body is ignored for GET requests."));
	}
	
	AppMobiOAuth.getProtectedData(parameters.service, parameters.url, parameters.id, parameters.method, parameters.body, parameters.headers);
};

AppMobi.addConstructor(function() {
    if (typeof AppMobi.oauth == "undefined") AppMobi.oauth = new AppMobi.OAuth();
});

/**
 * This class provides access to facebook.
 * @constructor
 */
AppMobi.Facebook = function() {

	/*var _willHandleUserTouchingLink = false;
    this.__defineGetter__("willHandleUserTouchingLink", function(){
        return _willHandleUserTouchingLink;
    });
    this.__defineSetter__("willHandleUserTouchingLink", function(val){
		AppMobi.exec("AppMobiFacebook.setWillHandleUserTouchingLink", val);		
        _willHandleUserTouchingLink = val;
    });*/
}

AppMobi.Facebook.prototype.login = function(permissions) {
    if( typeof( permissions ) == "undefined" ) permissions = "publish_stream,publish_actions,offline_access";
	AppMobiFacebook.login(permissions);
}

AppMobi.Facebook.prototype.logout = function() {
	AppMobiFacebook.logout();
}

AppMobi.Facebook.prototype.requestWithRestAPI = function(command, method, parameters) {
    if( typeof( parameters ) == "object" ) parameters = JSON.stringify( parameters);
	AppMobiFacebook.requestWithRestAPI([command, method, parameters]);
}

AppMobi.Facebook.prototype.requestWithGraphAPI = function(path, method, parameters) {
    if( typeof( parameters ) == "object" ) parameters = JSON.stringify( parameters);
	AppMobiFacebook.requestWithGraphAPI([path, method, parameters]);
}

AppMobi.Facebook.prototype.showAppRequestDialog = function(parameters) {
    if( typeof( parameters ) == "undefined" || typeof( parameters ) != "object" ) parameters = {};
    var paramstr = JSON.stringify(parameters);
	AppMobiFacebook.showAppRequestDialog(paramstr);
}

AppMobi.Facebook.prototype.showNewsFeedDialog = function(parameters) {
    if( typeof( parameters ) == "undefined" || typeof( parameters ) != "object" ) parameters = {};
    var paramstr = JSON.stringify(parameters);
	AppMobiFacebook.showNewsFeedDialog(paramstr);
}

AppMobi.Facebook.prototype.enableFrictionlessRequests = function() {
	AppMobiFacebook.enableFrictionlessRequests();
}

AppMobi.addConstructor(function() {
    if (typeof AppMobi.facebook == "undefined") AppMobi.facebook = new AppMobi.Facebook();
});

function getAppMobiObject()
{
	if( typeof AppMobi == "undefined" )
		return null;
	return AppMobi;
}

function getAppMobiDocument()
{
	return document;
}

/**
 * This class provides multi-touch support.
 * @constructor
 */
AppMobi.Multitouch = function() {
	
	this.targets = {};
	this.listeners = {};
	this.listeners.LAT = {}; //used to validate add/remove only - avoid removing for listener that hasnt been added
	this.listeners.length = 0;
	this.listeners.parent = this;
	this.listeners.add = function(type, target, listener) {
		if(!this[type]) {
			this[type] = [];
			this.length++;
		}
		this[type].push(target);
		
		if(!AppMobi.multitouch.listeners.LAT[type]) {
			AppMobi.multitouch.listeners.LAT[type] = [];
		}
		AppMobi.multitouch.listeners.LAT[type].push([target, listener]);
	}
	this.listeners.remove = function(type, target, listener) {
		//verify that the LAT contains a valid entry before proceeding with removal
		var valid = false;
		if(AppMobi.multitouch.listeners.LAT[type]) {
			for(var i=0;i<this.LAT[type].length;i++) {
				if(AppMobi.multitouch.listeners.LAT[type][i][0]==target && AppMobi.multitouch.listeners.LAT[type][i][1]==listener) {
					valid = true;
					break;
				}
			}
		}
		if(!valid) {
			//console.log('_AMMT_ '+ 'invalid request to remove '+type+' from '+target+', bailing');
			return;
		}
	
		if(this[type]) {
			var idx = this[type].indexOf(target);
			if(idx!=-1) {
				this[type].splice(idx,1);
				if(this[type].length==0) {
					delete this[type];
					this.length--;
				}
			}
		}
	}
	this.listeners.poll = function() {
		if(!AppMobi.multitouch.listeners.stopPolling) {
			//get data from native
			var dataset = eval(AppMobiDevice.getMultitouchData());
			AppMobi.multitouch.handleTouch(dataset);
			setTimeout(AppMobi.multitouch.listeners.poll, 25);
		}
	}
	this.listeners.dump = function() {
		var dump = 'listeners dump:';
		for(t in this) {
			if(typeof this[t] != 'function') {
				if(this[t] instanceof Array) {
					var l = '[';
					for(p in this[t]) {
						l+=p+':'+this[t][p].outerHTML+',';
					}
					l+=']';
					dump+=t+':'+l+'~';
					
				}
			}
		}
		console.log(dump);
	}
	
	this.touchTargets = {};
	this.touches = {};
}

AppMobi.Multitouch.prototype.enable = function() {
	//replace existing listeners using the queue
    for(var index in Node.prototype.ELQueue) {
    	var EL = Node.prototype.ELQueue[index];
    	if(EL[0]=='add') {
    		Node.prototype.removeEventListener_original.call(EL[1], EL[2], EL[3], EL[4]);
			AppMobi.multitouch.add(EL[1], EL[2], EL[3], EL[4]);
    	} else {
			AppMobi.multitouch.remove(EL[1], EL[2], EL[3], EL[4]);
    	}
    }
    delete Node.prototype.ELQueue;

	//override Node.add/removeEventListener
	Node.prototype.addEventListener = function(type, listener, useCapture) {
		if(Node.prototype.ELOverrideList.indexOf(type)!=-1) {
			AppMobi.multitouch.add(this, type, listener, useCapture);
		} else {
			Node.prototype.addEventListener_original.call(this, type, listener, useCapture);
		}
	}
	Node.prototype.removeEventListener = function(type, listener, useCapture) {
		if(Node.prototype.ELOverrideList.indexOf(type)!=-1) {
			AppMobi.multitouch.remove(this, type, listener, useCapture);
		} else {
			Node.prototype.removeEventListener_original.call(this, type, listener, useCapture);
		}
	}	
	
	//start the native side
	AppMobiDevice.enableMultitouch();
	//start polling
	AppMobi.multitouch.listeners.stopPolling = false;
	AppMobi.multitouch.listeners.poll();
}

/**
 * @param {element} target
 * @param {string} type ('touchstart', 'touchend', 'touchmove')
 * @param {function} listener
 * @param {boolean} useCapture
 */
AppMobi.Multitouch.prototype.add = function(target, type, listener, useCapture) {
	//AppMobi.debug.log('AppMobi.Multitouch.prototype.add called:target:'+target+' type:'+type);
	target.addEventListener('appMobi.'+type, listener, useCapture);
	this.listeners.add(type, target, listener);
	//this.listeners.dump();//debug
}

/**
 * @param {element} target
 * @param {string} type ('touchstart', 'touchend', 'touchmove')
 * @param {function} listener
 * @param {boolean} useCapture
 */
AppMobi.Multitouch.prototype.remove = function(target, type, listener, useCapture) {
	target.removeEventListener('_AMMT_ '+ 'appMobi.'+type, listener, useCapture);
	this.listeners.remove(type, target, listener);
}

AppMobi.Multitouch.prototype.sendTouch = function(data, target, touches, targetTouches, changedTouches) {
	var e = document.createEvent('Events');
	e.initEvent('appMobi.'+data.type, true, true);
    e.touches = touches;
    e.changedTouches = changedTouches;
    e.targetTouches = targetTouches;
	target.dispatchEvent(e);
	//AppMobi.debug.log("sent "+data.type+" to "+target.id+" touches:"+touches.length+" targetTouches:"+targetTouches.length+" changedTouches:"+changedTouches.length);
}

AppMobi.Multitouch.prototype.handleTouch = function(dataset) {
	try{
		for(var i=0;i<dataset.length;i++) {
			var data = dataset[i];
	
			//AppMobi.debug.log("{id:"+data.id+", x:"+data.x+", y:"+data.y+", type:"+data.type+"}");
			//AppMobi.debug.log("{id:"+data.id+", x:"+data.x+", screen.width:"+screen.width+", AppMobi.device.width:"+AppMobi.device.width+", window.innerWidth:"+window.innerWidth+", type:"+data.type+"}");
		
			//adjust x/y for scale
			//scaleX = screen.width/AppMobi.device.width;
			//scaleY = screen.height/AppMobi.device.height;
			scaleX = window.innerWidth/AppMobi.device.width;
			scaleY = window.innerHeight/AppMobi.device.height;
			data.x *=scaleX;
			data.y *=scaleY;
			
			//get touch and target
			var target, touch;
			if(data.type=='touchstart') {
				//get target at point
			    target = document.elementFromPoint(data.x, data.y);
			    //associate the target with id - the target will be re-used for touchmoves and touchend
			    this.touchTargets[data.id] = target;
				//build touch
				var touch = {};
				touch={};
				touch.identifier=data.id;
				this.touches[data.id] = touch;
			} else {
				target = this.touchTargets[data.id];
				touch = this.touches[data.id];
			}
			
			//if touch is undefined, we got a touchmove or touchend without getting a touchstart first
			if(touch==undefined) {
				//AppMobi.debug.log(data.type+" without touchstart");
				continue;
			}
			
			//if this is a touchmove but x/y hasnt changed, skip it
			if(data.type=='touchmove'&&touch.pageX==data.x&&touch.pageY==data.y) {
				continue;
			}
			
			//only one set of coords - assign to all 3 in touch
			touch.pageX=data.x;
			touch.pageY=data.y;
			touch.clientX=data.x;
			touch.clientY=data.y;
			touch.screenX=data.x;
			touch.screenY=data.y;
	
			//retarget from HTML to document
		    if(target.tagName=='HTML') {
		    	var tmp = target;
		    	target = document;
		    	document.clientWidth = tmp.clientWidth;
		    	document.clientHeight = tmp.clientHeight;
		    }
			
			//check if target has an inline event handler - if so, replace it
			if(target && target['on'+data.type]) {
				var orig = target['on'+data.type];
				target['on'+data.type] = null;
				AppMobi.multitouch.add(target, data.type, orig, false);
			}
			
		    //true if the target is not null and is registered as a listener
		    var targetIsValid = target && this.listeners[data.type] && this.listeners[data.type].indexOf(target)!=-1;
		    
			//AppMobi.debug.log("{touch.identifier:"+touch.identifier+", touch.pageX:"+touch.pageX+", touch.pageY:"+touch.pageY+", target:"+(target?target.id:"undefined")+", targetIsValid:"+targetIsValid+"}");

		    
		    //remove the association between id and target
			if(data.type=='touchend') {
				delete this.touchTargets[data.id];
				delete this.touches[data.id];
			}	    
	
			//only send if target is valid
		    if(target || targetIsValid) {
		    	
		    	//touches and targetTouches
				var touches = [];
				var targetTouches = [];
		    	for(var t in this.touches) {
		    		var tt = this.touches[t];
		    		touches.push(tt);
		    		if(this.touchTargets[tt.identifier]==target) {
		    			targetTouches.push(tt);
		    		}
		    	}
		    	//changedTouches
				var changedTouches = [touch];
				
			    this.sendTouch(data, target, touches, targetTouches, changedTouches);
		    }
		}
	} catch(e) {
		//sometimes an exception occurs - handle it so that touches dont break
		AppMobi.debug.log("a multiTouch error occurred: "+e.message);
	}
}

AppMobi.addConstructor(function() {
	//only enable if native side has multiTouch
	if(AppMobiDevice.enableMultitouch) {
    	if (typeof AppMobi.multitouch == "undefined") AppMobi.multitouch = new AppMobi.Multitouch();
    } else {
		Node.prototype.addEventListener = Node.prototype.addEventListener_original;
		Node.prototype.removeEventListener = Node.prototype.removeEventListener_original;
		delete Node.prototype.ELQueue;		
    }
});

//override list
Node.prototype.ELOverrideList = ['touchstart', 'touchmove', 'touchend'];
Node.prototype.ELQueue = [];

Node.prototype.addEventListener_original = Node.prototype.addEventListener;
Node.prototype.removeEventListener_original = Node.prototype.removeEventListener;

//override Node.add/removeEventListener
Node.prototype.addEventListener = function(type, listener, useCapture) {
	if(Node.prototype.ELOverrideList.indexOf(type)!=-1) {
		Node.prototype.ELQueue.push(['add', this, type, listener, useCapture]);
	}
	Node.prototype.addEventListener_original.call(this, type, listener, useCapture);
}
Node.prototype.removeEventListener = function(type, listener, useCapture) {
	if(Node.prototype.ELOverrideList.indexOf(type)!=-1) {
		Node.prototype.ELQueue.push(['remove', this, type, listener, useCapture]);
	}
	Node.prototype.removeEventListener_original.call(this, type, listener, useCapture);
}
