//allow binding
Function.prototype.bind = function(bind) {
	var self = this;
	return function(){
		var args = Array.prototype.slice.call(arguments);
		return self.apply(bind || null, args);
	};
};

AppMobiImage = function(w, h) {
	var _id = DirectCanvas2DContext.createImage(w?w:-1, h?h:-1);
	//console.log('created AppMobiImage w id:' + _id);
	var retval = {
		id:_id,
		type:"AppMobiImage",
		set src (s) {
			//console.log('set src on AppMobiImage w id:' + _id + 'w:' + s);
			DirectCanvas2DContext.setImageSource(this.id, s);
			if(this.onload)this.onload();
		},
		get src () {
			return DirectCanvas2DContext.getImageSource(this.id);
		},
		set height (h) {
			_h = h;
			DirectCanvas2DContext.setImageHeight(this.id, h);
		},
		get height () {
			return DirectCanvas2DContext.getImageHeight(this.id);
		},
		set width (w) {
			_w = w;
			DirectCanvas2DContext.setImageWidth(this.id, w);
		},
		get width () {
			return DirectCanvas2DContext.getImageWidth(this.id);
		},
		get complete() {
			return true;
		},
		hintLoad: function() {
			DirectCanvas2DContext.hintLoadImage(this.id);
		},
		hintUnload: function() {
			DirectCanvas2DContext.hintUnloadImage(this.id);
		}
	}
	return retval;
};

Image = AppMobiImage;

AppMobiImageData = function(id) {
	this.id = id;
	this.create = function(w, h) {
		this.width = w;
		this.height = h;
		this.id = DirectCanvas2DContext.createImageData(this.id,w,h);
		this.makeData(w*h*4);
	};
	this.copyCreate = function(orig) {
		this.width = orig.w;
		this.height = orig.h;
		this.id = DirectCanvas2DContext.createImageDataWithImageData(this.id,orig.id);
		this.makeData(this.width*this.height*4);
	};
	this.getCreate = function(x, y, w, h) {
		this.width = w;
		this.height = h;
		this.id = DirectCanvas2DContext.getImageData(this.id,x,y,w,h);
		this.makeData(w*h*4);
	};
	this.makeData = function(length) {
		this.data = {};
		this.data.length = length;
		for(var i=0;i<length;i++) {
			(function(id, idx, data) {
					data.__defineGetter__(idx, function() {
						return DirectCanvas2DContext.getImageDataPixelComponent(id, idx);
					});
					data.__defineSetter__(idx, function(value) {
						DirectCanvas2DContext.setImageDataPixelComponent(id, idx, value);
					});
			})(this.id, i, this.data);
		}
	};
};

AppMobiCanvasProto = {

}

function makeAppMobiCanvas(canvasId) {
	//private vars
	var listeners = {};//for canvas
	
	return {
		id: canvasId,
		__proto__: AppMobiCanvasProto,
		type: "AppMobiCanvas",
		twoDContext: {
			drawCmds: [],
			drawOffsets: [],
			drawValues: [],
			drawBuffer:false,
			get id () {
				return canvasId
			},
			//access to View properties
			set width (value) {
				DirectCanvas2DContext.setViewWidth(this.id, parseInt(value));
			},
			get width () {
				var w = DirectCanvas2DContext.getViewWidth(this.id);
				//return default is uninitialized
				if(w == -1) w = AppMobiDevice.getDisplayWidth();
				return w;
			},
			set height (value) {
				DirectCanvas2DContext.setViewHeight(this.id, parseInt(value));
			},
			get height () {
				var w = DirectCanvas2DContext.getViewHeight(this.id);
				//return default is uninitialized
				if(w == -1) w = AppMobiDevice.getDisplayHeight();
				return w;
			},
			
			//iOS DirectCanvas compatibility
			set globalScale (value) {
				DirectCanvas2DContext.setGlobalScale(this.id, value);
			},
			get globalScale () {
				return DirectCanvas2DContext.getGlobalScale(this.id);
			},
			set clearStyle (value) {
				DirectCanvas2DContext.setClearStyle(this.id, value);
			},
			get clearStyle () {
				return DirectCanvas2DContext.getClearStyle(this.id);
			},
			scale: function(s) {
				DirectCanvas2DContext.scale(this.id, s);
			},
			present: function() {
				if(this.drawBuffer) {
					for(i=0;i<this.drawValues.length;i++) {
						//var tmp = this.drawValues[i];
						this.drawValues[i]+=.00001;
						//console.log(tmp+":"+this.drawValues[i]);
					}
					DirectCanvas2DContext.present(this.id, this.drawCmds, this.drawOffsets, this.drawValues);
					this.drawCmds = [];
					this.drawOffsets = [];
					this.drawValues = [];
				} else {
					DirectCanvas2DContext.present(this.id);
				}
			},
			clear: function() {
				DirectCanvas2DContext.clear(this.id);
			},
			detach: function() {
				DirectCanvas2DContext.detach(this.id);
			},
			setFPS: function(fps) {
				DirectCanvas2DContext.setFPS(this.id, fps);
			},
			drawImageTile: function(img, dx, dy, t, dw, dh, flipX, flipY, alpha) {
				if(this.drawBuffer) {
					if(arguments.length == 4) {
						this.drawCmds.push('tile');
						this.drawOffsets.push(4);
						this.drawValues.push(img.id,dx,dy,t);
					} else if(arguments.length == 9) {
						this.drawCmds.push('tile');
						this.drawOffsets.push(9);
						this.drawValues.push(img.id,dx,dy,t,dw,dh,flipX?1:0,flipY?1:0,alpha);
					} else {
						//error
					}
				} else {
					if(arguments.length == 4) {
						//t is alpha in this case
						DirectCanvas2DContext.drawImageTile(this.id, img.id, dx, dy, t);
					} else if(arguments.length == 9) {
						DirectCanvas2DContext.drawImageTile2(this.id, img.id, dx, dy, t, dw, dh, flipX, flipY, alpha);
					} else {
						//error
					}
				}
			},
			drawRotatedImage: function(img, dx, dy, t, dw, dh, flipX, flipY, alpha, tx, ty, angle) {
				if(this.drawBuffer) {
					this.drawCmds.push('rot');
					this.drawOffsets.push(12);
					this.drawValues.push(img.id,dx,dy,t,dw,dh,flipX?1:0,flipY?1:0,alpha,tx,ty,angle);
				} else {
					DirectCanvas2DContext.drawRotatedImage(this.id, img.id, dx, dy, t, dw, dh, flipX, flipY, alpha, tx, ty, angle);
				}
			},
			
			// state
			save: function() {
				DirectCanvas2DContext.save(this.id);
			},
			restore: function() {
				DirectCanvas2DContext.restore(this.id);
			},

			// transformations
			scale: function(x,y) {
				DirectCanvas2DContext.scale(this.id, x, y);
			},
			rotate: function(angle) {
				DirectCanvas2DContext.rotate(this.id, angle);
			},
			translate: function(x,y) {
				DirectCanvas2DContext.translate(this.id, x, y);
			},
			transform: function(a,b,c,d,e,f) {
				DirectCanvas2DContext.transform(this.id, a,b,c,d,e,f);
			},
			setTransform: function(a,b,c,d,e,f) {
				DirectCanvas2DContext.setTransform(this.id, a,b,c,d,e,f);
			},
			
			// compositing
			set globalAlpha (value) {
				DirectCanvas2DContext.setGlobalAlpha(this.id, value);
			},
			get globalAlpha () {
				return DirectCanvas2DContext.getGlobalAlpha(this.id);
			},
			set globalCompositeOperation (value) {
				DirectCanvas2DContext.setGlobalCompositeOperation(this.id, value);
			},
			get globalCompositeOperation () {
				return DirectCanvas2DContext.getGlobalCompositeOperation(this.id);
			},

	        // colors and styles
			set strokeStyle (rgb) {
				DirectCanvas2DContext.setStrokeStyle(this.id, rgb);
			},
			get strokeStyle () {
				return DirectCanvas2DContext.getStrokeStyle(this.id);
			},
			set fillStyle (rgb) {
				DirectCanvas2DContext.setFillStyle(this.id, rgb);
			},
			get fillStyle () {
				return DirectCanvas2DContext.getFillStyle(this.id);
			},

			// rects
			clearRect: function(x, y, w, h) {
				DirectCanvas2DContext.clearRect(this.id, x, y, w, h);
			},
			fillRect: function(x, y, w, h) {
				DirectCanvas2DContext.fillRect(this.id, x, y, w, h);
			},
			strokeRect: function(x, y, w, h) {
				DirectCanvas2DContext.strokeRect(this.id, x, y, w, h);
			},
			
			// path API
			beginPath: function() {
				DirectCanvas2DContext.beginPath(this.id);
			},
			closePath: function() {
				DirectCanvas2DContext.closePath(this.id);
			},
			moveTo: function(x, y) {
				DirectCanvas2DContext.moveTo(this.id, x, y);
			},
			lineTo: function(x, y) {
				DirectCanvas2DContext.lineTo(this.id, x, y);
			},
			quadraticCurveTo: function(cpx, cpy, x, y) {
				DirectCanvas2DContext.quadraticCurveTo(this.id, cpx, cpy, x, y);
			},
			bezierCurveTo: function(cp1x, cp1y, cp2x, cp2y, x, y) {
				DirectCanvas2DContext.bezierCurveTo(this.id, cp1x, cp1y, cp2x, cp2y, x, y);
			},
			arcTo: function(x1, y1, x2, y2, radius) {
				DirectCanvas2DContext.arcTo(this.id, x1, y1, x2, y2, radius);
			},
			rect: function(x, y, w, h) {
				DirectCanvas2DContext.rect(this.id, x, y, w, h);
			},
			arc: function(x, y, radius, startAngle, endAngle, anticlockwise) {
				DirectCanvas2DContext.arc(this.id, x, y, radius, startAngle, endAngle, anticlockwise);
			},
			fill: function() {
				DirectCanvas2DContext.fill(this.id);
			},
			stroke: function() {
				DirectCanvas2DContext.stroke(this.id);
			},
			clip: function() {
				DirectCanvas2DContext.clip(this.id);
			},
			isPointInPath: function(x, y) {
				DirectCanvas2DContext.isPointInPath(this.id, x, y);
			},
			
			// text
			set font (value) {
				DirectCanvas2DContext.setFont(this.id, value);
			},
			get font () {
				return DirectCanvas2DContext.getFont(this.id);
			},
			set textAlign (value) {
				DirectCanvas2DContext.setTextAlign(this.id, value);
			},
			get textAlign () {
				return DirectCanvas2DContext.getTextAlign(this.id);
			},
			set textBaseline (value) {
				DirectCanvas2DContext.setTextBaseline(this.id, value);
			},
			get textBaseline () {
				return DirectCanvas2DContext.getTextBaseline(this.id);
			},
			fillText: function(text, x, y, maxWidth) {
				if(arguments.length == 3) {
					DirectCanvas2DContext.fillText(this.id, text, x, y);
				} else if(arguments.length == 4) {
					DirectCanvas2DContext.fillText(this.id, text, x, y, maxWidth);
				}
			},
			strokeText: function(text, x, y, maxWidth) {
				if(arguments.length == 3) {
					DirectCanvas2DContext.strokeText(this.id, text, x, y);
				} else if(arguments.length == 4) {
					DirectCanvas2DContext.strokeText(this.id, text, x, y, maxWidth);
				}
			},
			
			// drawing images/canvases/videos
			drawImage: function(img, sx, sy, sw, sh, dx, dy, dw, dh) {
				if(this.drawBuffer) {
					if(img && img.type=="AppMobiImage") {//validate
						if(arguments.length == 3) {
							this.drawCmds.push('img');
							this.drawOffsets.push(3);
							this.drawValues.push(img.id,sx,sy);
						} else if(arguments.length == 5) {
							this.drawCmds.push('img');
							this.drawOffsets.push(5);
							this.drawValues.push(img.id,sx,sy,sw,sh);
						} else if(arguments.length == 9) {
							this.drawCmds.push('img');
							this.drawOffsets.push(9);
							this.drawValues.push(img.id,sx,sy,sw,sh,dx,dy,dw,dh);
						} else {
							//error
							console.log('drawImage called w/ invalid number of args:'+arguments.length);
						}
					} else if (img && img.type=="AppMobiCanvas"){
						if(arguments.length == 3) {
							this.drawCmds.push('cvs');
							this.drawOffsets.push(3);
							this.drawValues.push(img.id,sx,sy);
						} else if(arguments.length == 5) {
							this.drawCmds.push('cvs');
							this.drawOffsets.push(5);
							this.drawValues.push(img.id,sx,sy,sw,sh);
						} else if(arguments.length == 9) {
							this.drawCmds.push('cvs');
							this.drawOffsets.push(9);
							this.drawValues.push(img.id,sx,sy,sw,sh,dx,dy,dw,dh);
						} else {
							//error
							console.log('drawImage called w/ invalid number of args:'+arguments.length);
						}
					} else {
						//error
						var description = "";
						for(p in img) {
							description+=p+":"+img[p]+"~";
						}
						console.log('drawImage called w/ invalid object:'+description);
					}
				} else {
					//fixme
					if(img && img.type=="AppMobiImage") {//validate
						if(arguments.length == 3) {
							DirectCanvas2DContext.drawImage(this.id, img.id, sx, sy);
						} else if(arguments.length == 5) {
							DirectCanvas2DContext.drawImage(this.id, img.id, sx, sy, sw, sh);
						} else if(arguments.length == 9) {
							DirectCanvas2DContext.drawImage(this.id, img.id, sx, sy, sw, sh, dx, dy, dw, dh);
						} else {
							//error
							console.log('drawImage called w/ invalid number of args:'+arguments.length);
						}
					} else if (img && img.type=="AppMobiCanvas"){
						if(arguments.length == 3) {
							DirectCanvas2DContext.drawCanvas(this.id, img.id, sx, sy);
						} else if(arguments.length == 5) {
							DirectCanvas2DContext.drawCanvas(this.id, img.id, sx, sy, sw, sh);
						} else if(arguments.length == 9) {
							DirectCanvas2DContext.drawCanvas(this.id, img.id, sx, sy, sw, sh, dx, dy, dw, dh);
						} else {
							//error
							console.log('drawImage called w/ invalid number of args:'+arguments.length);
						}
					} else {
						//error
						var description = "";
						for(p in img) {
							description+=p+":"+img[p]+"~";
						}
						console.log('drawImage called w/ invalid object:'+description);
					}
				}
			},
			
			// pixel manipulation
			createImageData: function(sw, sh) {
				var data = new AppMobiImageData(this.id);
				if(arguments.length == 1) {
					//arguments[0] is an ImageData
					data.copyCreate(sw);
				} else if(arguments.length == 2) {
					data.create(sw,sh);
				} else {
					//error
				}
				return data;
			},
			getImageData: function(sx, sy, sw, sh) {
				var data = new AppMobiImageData(this.id);
				data.getCreate(sx, sy, sw, sh);
				return data;
			},
			putImageData: function(imagedata, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight) {
				if(! imagedata instanceof AppMobiImageData) {//validate
					//error
					return;
				}
				if(arguments.length == 3) {
					DirectCanvas2DContext.putImageData(this.id, imagedata.id, dx, dy);
				} else if(arguments.length == 7) {
					DirectCanvas2DContext.putImageDataWithDirtyRegion(this.id, imagedata.id, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight);
				} else {
					//error
				}
			},
			drawFont: function() {
				//fixme
				console.log('drawFont called');
			},
				
/*

needed for backwards compatibility with iOS DirectCanvas:
drawFont*(Font, text, dx, dy, alignment) - for Impact compatibility, will be removed (old comment - not adding to Android unless it is found to be needed)


		  // compositing
//		public void setGlobalCompositeOperation(String value);// (default source-over)
//		public String getGlobalCompositeOperation();
        
        // colors and styles
//        CanvasGradient createLinearGradient(int x0, int y0, int x1, int y1);
//        CanvasGradient createRadialGradient(int x0, int y0, int r0, int x1, int y1, int r1);
//        CanvasPattern createPattern(HTMLImageElement image, String repetition);
//        CanvasPattern createPattern(HTMLCanvasElement image, String repetition);
//        CanvasPattern createPattern(HTMLVideoElement image, String repetition);

		  // line caps/joins
//		public void setLineWidth(int value);// (default 1)
//		public int getLineWidth();
//		public void setLineCap(String value);// "butt", "round", "square" (default "butt")
//		public String getLineCap();
//		public void setLineJoin(String value);// "round", "bevel", "miter" (default "miter")
//		public String getLineJoin();
//		public void setMiterLimit(int value);// (default 10)
//		public int getMiterLimit();

        // shadows
//		public void setShadowOffsetX(int value); // (default 0)
//		public int getShadowOffsetX();
//		public void setShadowOffsetY(int value); // (default 0)
//		public int getShadowOffsetY();
//		public void setShadowBlur(int value); // (default 0)
//		public int getShadowBlur();
//		public void setShadowColor(String value); // (default transparent black)
//		public String getShadowColor();

		// Focus management
//		boolean drawFocusRing(Element element);
//		boolean drawFocusRing(Element element, boolean canDrawCustom);

		// Caret and selection management
//		int caretBlinkRate();
//		boolean setCaretSelectionRect(Element element, int x, int y, int w, int h);
		
		  // text
//		TextMetrics measureText(String text);

		// drawing images/canvases/videos
//		void drawCanvas(String id, int dx, int dy);
//		void drawCanvas(String id, int dx, int dy, int dw, int dh);
//		void drawCanvas(String id, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh);
//		void drawVideo(String id, int dx, int dy);
//		void drawVideo(String id, int dx, int dy, int dw, int dh);
//		void drawVideo(String id, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh);

*/					
		},

		getContext: function(){ 
			return this.twoDContext;
		},
				
		addEventListener: function(type, listener, useCapture){
			if(listeners[type]) {
				listeners[type].push(listener);
			} else {
				listeners[type] = [listener];
			}
		},
		removeEventListener: function(type, listener, useCapture){
			if(listeners[type]) {
				listeners[type].pop(listener);
			}
			if(listeners[type].length==0) {
				delete listeners[type];
			}
		},
		fireEvent: function(event) {
			var listenersForType = listeners[event.type];
			if(listenersForType) {
				for(listener in listenersForType) {
					listenersForType[listener](event);
				}
			}
		},
		dispatchEvent: function(event){this.fireEvent(event);},
		isnative: true//internal only
	}
}
AppMobi = function() {

	setTimeout('AppMobi.context.messagePump();', 150);	

	//parse querystring to get app base location
	var qs = location.href.split('?')[1];
	var qsargs = qs.split('&');
	var qsNVPs = {};
	for (var arg in qsargs) { 
		var nvp = qsargs[arg].split('=');
		qsNVPs[nvp[0]]=nvp[1];
	}	
	var _base = "http://localhost:58888/"+qsNVPs['appName']+"/"+qsNVPs['appRelease']+"/";
	
	return {
		//app base
		base: _base,
	
		//event hook callbacks: AppMobi.wasShown, AppMobi.willBeHidden
		wasShown: function(){},
		willBeHidden: function(){},
		updateFPS: function(fps){},
		
		platform: 'Android',
		isnative: true,
		
		// AppMobi.context provides basic utility functions, such as timers
		// and device properties
		context: {
			landscapeMode: function(){},//read only
			screenWidth: function(){},//read only
			devicePixelRatio: function(){},//read only
			screenHeight: function(){},//read only
			userAgent: function(){},//read only
			openURL: function(url){},//prompts user to open url in associated application
			executeJavascriptInWebView: function(script){},//this is a bridge function which allows the DirectCanvas component to execute the provided javascript in the appMobi component.
			include: function(){},//include a script - this is a placeholder and is reassigned below
			startBackgroundSound: function(relativeUrl, looping){
				AppMobiPlayer.startAudio(relativeUrl, looping);
			},
			toggleBackgroundSound: function(){
				AppMobiPlayer.toggleAudio();
			},
			stopBackgroundSound: function(){
				AppMobiPlayer.stopAudio();
			},
			playSound: function(relativeUrl){
				AppMobiPlayer.playSoundDC(relativeUrl);
			},
			pauseSound: function(relativeUrl){
				AppMobiPlayer.pauseSoundDC(relativeUrl);
			},
			loadSound: function(relativeUrl){
				AppMobiPlayer.loadSound(relativeUrl);
			},
			loadPolySound: function(relativeUrl, count){
				AppMobiPlayer.loadSound(relativeUrl, count);
			},
			hide: function(){},
			hideLoadingScreen: function(){},
			log: function(msg){
				AppMobiDebug.log(msg);
			},
			setTimeout: function(){},
			setInterval: function(){},
			clearInterval: function(){},
			clearTimeout: function(){},
			cancelAllTimers: function(){},
			messageCounter:0,
			messagePump: function() {
				var script = DirectCanvas.messagePump();
				if( script ) {
					//var msgId = this.messageCounter++;
					//AppMobiDebug.log('***got message '+msgId+':'+script,"");
					//try { eval(script+";AppMobiDebug.log('***evaled msg "+msgId+"');"); } catch(e) { AppMobiDebug.log(e.message); };
					try { eval(script); } catch(e) { console.log(e); };
				}
				setTimeout('AppMobi.context.messagePump();', 15);
			}
		},
		
		//setup webview - provides access to parent AppMobiWebView
		webview: {
			//webview.execute to execute javascript in webview
			execute: function(js){
				DirectCanvas.execute(js);
			}
		},
		
		//Audio should be available by default in the WebView
		//need to test the interface:
			/*
			loop - read/write
			volume - read/write
			currentTime - read/write
			play()
			pause()
			load()
			canPlayType(mimeType)
			addEventListener(type, func, shouldBubble*) *ignored
			removeEventListener() - stub
			*/		
		
		canvas: makeAppMobiCanvas(0),
		box2d: {

			AABB: function() {
				// bounding box is not needed to create the world in this version of box2d
				// implement if needed
			},
			Body: function(_bdef, _world, _id) {
				if(_id==undefined) {  
					this.id=DirectBox2d.BodyCreate(_bdef.id, _world.id);
				} else {
					this.id=_id;
					//console.log("Body id= "+_id);
				}
			}, 
			BodyDef: function() {
				this.id=DirectBox2d.BodyDefCreate();
				// if any property not changed can set default value here
				//this.type= ...
			},
			Contact: function(_id) {
				this.id=_id;
			},
			ContactListener: function() {
				this.id=DirectBox2d.ContactListenerCreate();
			},
			Fixture: function(_id) {
				if(_id==undefined) {  
					this.id=DirectBox2d.FixtureCreate();
				} else {
					this.id=_id;
				}
			},
			FixtureDef: function() {
				this.id=DirectBox2d.FixtureDefCreate();
			},
			CircleShape: function(_id) {
				if(_id==undefined) {  
					this.id=DirectBox2d.CircleShapeCreate();
				} else {
					this.id=_id;
				}
			},
			PolygonShape: function() {
				this.id=DirectBox2d.PolygonShapeCreate();
			},
			Vec2: function(_x, _y, _id) {
				this.x=_x;
				this.y=_y;
				if(_id==undefined) {  
					this.id=DirectBox2d.Vec2Create(_x, _y);
				} else {
					this.id=_id;
				}
			},
			World: function(g, s) {
				//:TODO - check for valid type of g
				//:TODO - implement default constructor for new World();
				this.gravity=g;
				this.doSleep=s;
				var ds=(s)?1:0;
				this.id=DirectBox2d.WorldCreate(g.id, ds);
			}
		}		
	}
}();

//script injection (for AppMobi.context.include)
AppMobi.injectQueue = [];
AppMobi.readyToInject = true;
AppMobi.makeReadyToInject = function() {
	AppMobi.readyToInject = true;
	AppMobi.doInject();
}
AppMobi.doInject = function() {
	//if not ready or queue is empty return
	if(!AppMobi.readyToInject || !AppMobi.injectQueue.length) {
		if(!AppMobi.injectQueue.length) console.log('queue is empty');
		if(!AppMobi.readyToInject) console.log('not ready to inject');
		return;
	}
	//set ready state
	AppMobi.readyToInject = false;
	//get next src from queue and make path absolute
	var src = AppMobi.injectQueue.shift();
	//create script tag with load listener
	var script = document.createElement("script");
	script.type = "text/javascript";
	script.onload = AppMobi.makeReadyToInject;
	script.src = src;
	document.body.appendChild(script);
}
AppMobi.inject = function(src){
	//add src to queue, call doInject
	AppMobi.injectQueue.push(src);
	AppMobi.doInject();
}
AppMobi.context.include = AppMobi.inject;

Canvas = AppMobi.canvas;

//override document element creation
document.createElementOriginal = document.createElement;
document.createElement = function(type){
	if(type=='canvas') {
		var ref = DirectCanvas2DContext.getNewCanvas();
		return makeAppMobiCanvas(ref);
	} else if (type=='script'){
		return document.createElementOriginal(document, type);
	} else {
		throw "directCanvas error: createElement is only valid for script in directCanvas";
	}
};
//disable some things for improved compatibility with iOS/Android directCanvas
Canvas.styleOriginal = Canvas.style;
Canvas.__defineGetter__("style", function(h){
	throw "directCanvas error: canvas.style is not a valid property in directCanvas";
});    
Canvas.__defineSetter__("style", function(h){
	throw "directCanvas error: canvas.style is not a valid property in directCanvas";
});

document.addEventListenerOriginal = document.addEventListener;
document.addEventListener = function( type, callback ) {
	if( type == 'DOMContentLoaded' ) {
		setTimeout( callback, 1 );
	} else {
		throw "directCanvas error: document.addEventListener is not available in directCanvas";
	}
}
document.getElementByIdOriginal = document.getElementById;
document.getElementById = function(id) {
	return null;
}
	
b2 = AppMobi.box2d;
b2.SCALE = 0.1;  // for Impact games 
b2Body = {
    b2_staticBody: 0,
    b2_kinematicBody: 1,
    b2_dynamicBody: 2
}

// Body
b2.Body.prototype = {
	get userData() {
		return JSON.parse(DirectBox2d.BodyGetUserData(this.id));
	}, 
	set userData(val) {
		DirectBox2d.BodySetUserData(this.id, JSON.stringify(val));
	}
}
b2.Body.prototype.CreateFixture = function(fixtureDef) {
	var fid = DirectBox2d.BodyCreateFixture(this.id, fixtureDef.id);
	var fix=new b2.Fixture(fid);
	return fix;
}
b2.Body.prototype.CreateFixture2 = function(shape, density) {
	if(density==undefined) {
		density=0;
	}
	var fid = DirectBox2d.BodyCreateFixture2(this.id, shape.id, density);
	var fix=new b2.Fixture(fid);
	return fix;
}
b2.Body.prototype.ApplyForce = function(force, point) {
	DirectBox2d.BodyApplyForce(this.id, force.id, point.id);
}
b2.Body.prototype.GetPosition = function() {
	var id = DirectBox2d.BodyGetPosition(this.id);
	var x=DirectBox2d.Vec2GetX(id);
	var y=DirectBox2d.Vec2GetY(id);
	var p=new b2.Vec2(x, y, id);
	return p;
}
b2.Body.prototype.GetRoundedAngle = function(x) {
	var ra = DirectBox2d.BodyGetRoundedAngle(this.id, x);
	return ra;
}
b2.Body.prototype.GetType = function() {
	var type = DirectBox2d.BodyGetType(this.id);
	return type;
}

// BodyDef
b2.BodyDef.prototype = {
	get type() {
		return DirectBox2d.BodyDefGetType(this.id);
	}, 
	set type(val) {
		DirectBox2d.BodyDefSetType(this.id, val);
	},
	get position() {
		var id = DirectBox2d.BodyDefGetPosition(this.id);
		var x=DirectBox2d.Vec2GetX(id);
		var y=DirectBox2d.Vec2GetY(id);
		var p=new b2.Vec2(x, y, id);
		return p;
	}, 
	set position(val) {
		DirectBox2d.BodyDefSetPosition(this.id, val.id);
	},
	get angle() {
		return DirectBox2d.BodyDefGetAngle(this.id);
	}, 
	set angle(val) {
		DirectBox2d.BodyDefSetAngle(this.id, val);
	},
	get linearVelocity() {
		var id = DirectBox2d.BodyDefGetLinearVelocity(this.id);
		var x=DirectBox2d.Vec2GetX(id);
		var y=DirectBox2d.Vec2GetY(id);
		var p=new b2.Vec2(x, y, id);
		return p;
	}, 
	set linearVelocity(val) {
		DirectBox2d.BodyDefSetLinearVelocity(this.id, val.id);
	},
	get angularVelocity() {
		return DirectBox2d.BodyDefGetAngularVelocity(this.id);
	}, 
	set angularVelocity(val) {
		DirectBox2d.BodyDefSetAngularVelocity(this.id, val);
	},
	get linearDamping() {
		return DirectBox2d.BodyDefGetLinearDamping(this.id);
	}, 
	set linearDamping(val) {
		DirectBox2d.BodyDefSetLinearDamping(this.id, val);
	},
	get angularDamping() {
		return DirectBox2d.BodyDefGetAngularDamping(this.id);
	}, 
	set angularDamping(val) {
		DirectBox2d.BodyDefSetAngularDamping(this.id, val);
	},
	get awake() {
		if(DirectBox2d.BodyDefGetAwake(this.id))
			return true;
		else
			return false;
	}, 
	set awake(val) {
		var awake=(vale)?1:0;
		DirectBox2d.BodyDefSetAllowSleep(this.id, awake);
	},
	get allowSleep() {
		if(DirectBox2d.BodyDefGetAllowSleep(this.id))
			return true;
		else
			return false;
	}, 
	set allowSleep(val) {
		if(val!=true)
			DirectBox2d.BodyDefSetAllowSleep(this.id, 0);
		else
			DirectBox2d.BodyDefSetAllowSleep(this.id, 1);
	},
	get userData() {
		//return DirectBox2d.BodyDefGetUserData(this.id);
		return JSON.parse(this._userData);
	},
	set userData(val) {
		this._userData = JSON.stringify(val);
		//DirectBox2d.BodyDefSetUserData(this.id, JSON.stringify(val));
	}
}
// Contact
b2.Contact.prototype.GetFixtureA = function() {
	var id = DirectBox2d.ContactGetFixtureA(this.id);
	return new b2.Fixture(id);
}
b2.Contact.prototype.GetFixtureB = function() {
	var id = DirectBox2d.ContactGetFixtureB(this.id);
	return new b2.Fixture(id);
}
// ContactListener
b2.ContactListener.prototype = {
	get BeginContact() {
		return undefined;
	}, 
	set BeginContact(val) {
		DirectBox2d.ContactListenerSetBeginContact(this.id, val.toString());
	}
}

// CircleShape
b2.CircleShape.prototype.SetRadius = function(_r) {
	DirectBox2d.CircleShapeSetRadius(this.id, _r);
}

// fixture
b2.Fixture.prototype.GetBody = function() {
	var id = DirectBox2d.FixtureGetBody(this.id);
	var b=new b2.Body(0, 0, id);
	return b;
}

// FixtureDef
b2.FixtureDef.prototype = {
	get shape() {
		var id = DirectBox2d.FixtureDefGetShape(this.id);
		var shapeType = DirectBox2d.FixtureDefGetShapeType(this.id);
		var shape;
		if(shapeType==0) {
			shape=new b2.CircleShape(id);
		} else {
			// shape=new b2.Shape(x, y, id);
		} 
		return shape;
	}, 
	set shape(val) {
		DirectBox2d.FixtureDefSetShape(this.id, val.id);
	},
	get friction() {
		return DirectBox2d.FixtureDefGetFriction(this.id);
	}, 
	set friction(val) {
		DirectBox2d.FixtureDefSetFriction(this.id, val);
	},
	get density() {
		return DirectBox2d.FixtureDefGetDensity(this.id);
	}, 
	set density(val) {
		DirectBox2d.FixtureDefSetDensity(this.id, val);
	},
	get restitution() {
		return DirectBox2d.FixtureDefGetRestitution(this.id);
	}, 
	set restitution(val) {
		DirectBox2d.FixtureDefSetRestitution(this.id, val);
	},
}
// PolygonShape

b2.PolygonShape.prototype.SetAsBox = function(hx, hy) {
	DirectBox2d.PolygonShapeSetAsBox(this.id, hx, hy);
}

// Vec2
b2.Vec2.prototype.SetZero = function() {
	this.x=0;
	this.y=0;
	DirectBox2d.Vec2SetZero(this.id);
}
b2.Vec2.prototype.Set = function(_x, _y) {
	this.x=_x;
	this.y=_y;
	DirectBox2d.Vec2Set(this.id, _x, _y);
}
b2.Vec2.prototype.GetNegative = function() {
	//DirectBox2d.Vec2GetNegative(this.id, _x, _y); // returning a string from Java and getting properties is an alternative implementation
	return new b2.Vec2(-this.x, -this.y);
}
b2.Vec2.prototype.Add = function(v2) {
	this.x += v2.x;
	this.y += v2.y;
	DirectBox2d.Vec2Add(this.id, v2.id);
}

b2.Vec2.prototype.Subtract = function(v2) {
	this.x -= v2.x;
	this.y -= v2.y;
	DirectBox2d.Vec2Sub(this.id, v2.id);
}
b2.Vec2.prototype.Subtract = function(m) {
	this.x *= v2.x;
	this.y *= v2.y;
	DirectBox2d.Vec2Mul(this.id, v2.id);
}
b2.Vec2.prototype.Length = function() {
	return Math.sqrt(this.x*this.x + this.y*this.y);
}
b2.Vec2.prototype.LengthSquared = function() {
	return this.x*this.x + this.y*this.y;
};
b2.Vec2.prototype.Normalize = function() {
	return DirectBox2d.Vec2Normalize(this.id);
}
b2.Vec2.prototype.IsValid = function() {
	return DirectBox2d.IsValid(this.id);
}
// World
b2.World.prototype.CreateBody = function(bodyDef) {
	var id=bodyDef.id;
	var bid=DirectBox2d.WorldCreateBody(this.id, id);
	if(bodyDef._userData != undefined) {
		DirectBox2d.BodySetUserData(bid, bodyDef._userData);
	}
	var body=new b2.Body(0, 0, bid);
	return body;
}
b2.World.prototype.DestroyBody = function(body) {
	var id=body.id;
	var bid=DirectBox2d.WorldDestroyBody(this.id, id);
}
b2.World.prototype.SetContactListener = function(cl) {
	var id=cl.id;
	DirectBox2d.WorldSetContactListener(this.id, id);
}
b2.World.prototype.Step = function(tick, vIterations, pIterations) {
	DirectBox2d.WorldStep(this.id, tick, vIterations, pIterations);
}

//================================================================
// DIRECT CANVAS TOUCH TRANSLATION
//================================================================	
Canvas._orig_addEventListener = Canvas.addEventListener;
Canvas._orig_removeEventListener = Canvas.removeEventListener;

Canvas.addEventListener = function(eventType, eventHandler, eventCapture)
{
	if(eventType in {"touchstart":'',"touchmove":'',"touchend":''}) {
		//AppMobi.webview.execute("AppMobi.multitouch.add(document, '"+eventType+"', AppMobi.canvas.forwardTouchEvent, false);");
		AppMobi.webview.execute("document.addEventListener('"+eventType+"', AppMobi.canvas.forwardTouchEvent, false);");
	}
	this._orig_addEventListener(eventType, eventHandler, eventCapture);
}

Canvas.removeEventListener = function(eventType, eventHandler, eventCapture)
{
	if(eventType in {"touchstart":'',"touchmove":'',"touchend":''})
		//AppMobi.webview.execute("AppMobi.multitouch.remove(document, '"+eventType+"', AppMobi.canvas.forwardTouchEvent, false);");
		AppMobi.webview.execute("document.removeEventListener('"+eventType+"', AppMobi.canvas.forwardTouchEvent, false);");
	this._orig_removeEventListener(eventType, eventHandler, eventCapture);
}

Canvas.setMouseCoords = function(orientation, deviceW, deviceH, webW, webH, webX, webY, eventType)
{
	//console.log("orientation:"+orientation+", deviceW:"+deviceW+", deviceH:"+deviceH+", webW:"+webW+", webH:"+webH+", webX:"+webX+", webY:"+webY+", eventType:"+eventType);
	
	var dwidth=deviceW, dheight=deviceH, cwidth, cheight, wscale, hscale, scaleFactor, wblackbar, hblackbar, mouseCoordX, mouseCoordY, realMouseX, realMouseY;
	
	cwidth = AppMobi.canvas.getContext('2D').width;//AppMobi.canvas.context.width;
	cheight = AppMobi.canvas.getContext('2D').height;//AppMobi.canvas.context.height;	

	wscale = dwidth/cwidth;
	hscale = dheight/cheight;
	
	if(hscale < wscale)
		scaleFactor = hscale;
	else
		scaleFactor = wscale;
	
	wblackbar = dwidth - (cwidth * scaleFactor);
	hblackbar = dheight - (cheight * scaleFactor);
	
	if(wblackbar > 0)
		wblackbar = wblackbar/2;
	if(hblackbar > 0)
		hblackbar = hblackbar/2;
		
	//console.log("dwidth:"+dwidth+", dheight:"+dheight+", cwidth:"+cwidth+", cheight:"+cheight+", wscale:"+wscale+", hscale:"+hscale+", scaleFactor:"+scaleFactor+", wblackbar:"+wblackbar+", hblackbar:"+hblackbar);

	webX = webX*(dwidth/webW);
	webY = webY*(dheight/webH);

	mouseCoordX = parseInt(webX * (cwidth/webW));
	mouseCoordY = parseInt(webY * (cheight/webH));
	
	realMouseX = parseInt((webX-wblackbar)/scaleFactor);
	realMouseY = parseInt((webY-hblackbar)/scaleFactor);
	
	//console.log("webX:"+webX+", webY:"+webY+", mouseCoordX:"+mouseCoordX+", mouseCoordY:"+mouseCoordY+", realMouseX:"+realMouseX+", realMouseY:"+realMouseY);

	if(realMouseX >= 0 && realMouseX < cwidth && realMouseY >= 0 && realMouseY < cheight)
	{
		//console.log("MOUSE X: "+realMouseX+ "     MOUSE Y: "+realMouseY);
		AppMobi.canvas.simulateTouchEvent(realMouseX,realMouseY,eventType);
	}
	
}

Canvas.simulateTouchEvent = function(touchX,touchY,touchType)
{
	//console.log(touchX+','+touchY+':'+touchType);
	var touchevt = new TouchEvent();
	touchevt.initEvent(touchType, true, true);
	touchevt.touches[0]={};
	touchevt.touches[0].pageX=touchX;//originalEvent.clientX;
	touchevt.touches[0].pageY=touchY;//originalEvent.clientY;
	touchevt.touches[0].target=undefined;//originalEvent.target;
	touchevt.changedTouches = touchevt.touches;//for jqtouch
	touchevt.targetTouches = touchevt.touches;//for jqtouch
	touchevt.touches[0].clientX=touchevt.touches[0].pageX; //compatibility code
	touchevt.touches[0].clientY=touchevt.touches[0].pageY; //compatibility code
	touchevt.target=undefined//originalEvent.target;
	AppMobi.canvas.dispatchEvent(touchevt);
}

TouchEvent = function() {
	this.bubbles = true;
	this.cancelBubble = false;
	this.cancelable = true;
	this.changedTouches = [];
	this.clipboardData = undefined;
	this.currentTarget = null;
	this.defaultPrevented = false;
	this.eventPhase = 0;
	this.returnValue = true;
	this.srcElement = "";//Canvas;
	this.target = "";//Canvas;
	this.touches = [];
	this.targetTouches = this.touches;
	this.timestamp = -1;
	this.type = "";
	return this;
}
TouchEvent.prototype = {};
TouchEvent.prototype.initEvent = function(type, bubbles, cancelable) {
	if(arguments.length>0 && typeof arguments[0]=="string")this.type = type;
	if(arguments.length>1 && typeof arguments[1]=="boolean")this.bubbles = bubbles;
	if(arguments.length>2 && typeof arguments[2]=="boolean")this.cancelable = cancelable;
}

function testTouch(e) {
	console.log(e.type+" x:"+e.touches[0].pageX+", y:"+e.touches[0].pageY);
}
//AppMobi.canvas.addEventListener('touchstart', testTouch, false);
//AppMobi.canvas.addEventListener('touchend', testTouch, false);
//AppMobi.canvas.addEventListener('touchmove', testTouch, false);	
//================================================================

// AppMobi.context.log only accepts one param; console.log accepts multiple params
// and joins them
console = {
	log: function() {
		var args = Array.prototype.join.call(arguments, ', ');
		AppMobi.context.log( args );
	}
};

//AppMobiDevice
devicePixelRatio = AppMobiDevice.getDisplayDensity();
/*
if( AppMobiDevice.getOrientation()==0 ) {
	innerWidth = AppMobiDevice.getDisplayWidth();
	innerHeight = AppMobiDevice.getDisplayHeight();
}
else {
	innerHeight = AppMobiDevice.getDisplayWidth();
	innerWidth = AppMobiDevice.getDisplayHeight();
}
*/
innerWidth = AppMobiDevice.getDisplayWidth();
innerHeight = AppMobiDevice.getDisplayHeight();
screen = {
	availWidth: innerWidth,
	availHeight: innerHeight
};

console.log('window.innerWidth:'+window.innerWidth);
console.log('window.innerHeight:'+window.innerHeight);
